#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
# Copyright (c) Ant <ant@anthive.com>

import pyglet
from pyglet.window import mouse
from pyglet import clock

import ngfp.config as cfg

from ngfp.active import ActiveAreaLeftMouseClickAction, ActiveAreaRightMouseClickAction, ActiveAreaMouseMoveAction
from ngfp.board import ClearSquareMarkers, ClearAllMarkers, ToggleMarker, DrawBoard
from ngfp.dialog import ShowHelp
from ngfp.labels import UpdateLabels
from ngfp.marbles import CheckMarbleChangeDirection, StopMarble
from ngfp.my_init import MyInitStuff


class Window(pyglet.window.Window):

    def __init__ (
            self,
            width,
            height,
            caption,
            resizable,
            fullscreen,
            visible,
            *args,
            **kwargs):


        super(Window, self).__init__(width, height, caption, resizable, fullscreen, visible, *args, **kwargs)

        self.set_visible(False)

        MyInitStuff (self)

        # put the gcube and cube someplace.
        # i may not need these eventually so not going to make this
        # a function for now...
        x_pos = cfg.img_pix * (cfg.game_cols+1)
        y_pos = 0
        self.gcube = pyglet.sprite.Sprite( self.gcube_image, batch=self.pointer_bottom_batch, x = x_pos, y = y_pos)
        self.top_sprites.append(self.gcube)
        x_pos = 0
        y_pos = 0
        self.cube = pyglet.sprite.Sprite( self.cube_image, batch=self.pointer_top_batch, x = x_pos, y = y_pos)
        self.top_sprites.append(self.cube)

        self.picked_up = False
        self.picked_up_window_square = -1
        self.picked_up_sprite = pyglet.sprite.Sprite( self.game_bg_image, batch=self.pointer_top_batch, x = 0, y = 0)
        self.picked_up_sprite.visible = False
        self.picked_up_sprite.opacity = 150


    def on_draw(self):
        self.render()


    def on_close(self):
        exit()


    def on_mouse_press(self, x, y, button, modifiers):

        img_pix = cfg.img_pix
        x_win = x // img_pix
        x_rec = x_win * img_pix
        y_win = y // img_pix
        y_rec = y_win * img_pix
        win_pos = (y_win * self.window_cols) + x_win

        if button == mouse.LEFT:
#            print("The LEFT mouse button was pressed.", x, x_rec, x_win, y, y_rec, y_win, win_pos)
            ActiveAreaLeftMouseClickAction(self, x, x_rec, y, y_rec, win_pos)
        elif button == mouse.MIDDLE:
#            print("The MIDDLE mouse button was pressed.", x, x_rec, x_win, y, y_rec, y_win, win_pos)
            pass
        elif button == mouse.RIGHT:
#            print("The RIGHT mouse button was pressed.", x, x_rec, x_win, y, y_rec, y_win, win_pos)
            ActiveAreaRightMouseClickAction(self, x, x_rec, y, y_rec, win_pos)


    def on_mouse_release(self, x, y, button, modifiers):
#        print ("The mouse was released")
        pass


    def on_mouse_motion(self, x, y, dx, dy):

        img_pix = cfg.img_pix
        x_win = x // img_pix
        x_rec = x_win * img_pix
        y_win = y // img_pix
        y_rec = y_win * img_pix
        win_pos = (y_win * self.window_cols) + x_win

        if (win_pos in self.guess_active_squares):
            self.mouse_win_pos = win_pos

        if (self.picked_up):
            ActiveAreaMouseMoveAction(self, x, x_rec, y, y_rec, win_pos)
        else:
            pass


    def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
        pass


    def on_mouse_leave(self, x, y):
        pass


    def on_mouse_enter(self, x, y):
        pass


    def on_key_press(self, symbol, modifiers):

        self.keys_held.append(symbol)
        if ((symbol == pyglet.window.key.ESCAPE) or (symbol == pyglet.window.key.Q)): # [ESC] or [Q]
#            print ("The 'ESC' or 'Q' key was pressed")
            exit()
        elif ((symbol == pyglet.window.key.F1) or
            (symbol == pyglet.window.key.QUESTION) or
            (symbol == pyglet.window.key.H)):
            ShowHelp (self)
#            print ("The 'F1', 'H', or '?' key was pressed ")
            pass
        elif symbol == pyglet.window.key.F2:
#            print ("The 'F2' key was pressed, show board ", cfg.show_board)
            # after the initial showing of the background we
            # don't ever need to see the background again so 
            # only toggle between the game board and the guess 
            # board (0 or 1)...
            cfg.show_board = (cfg.show_board + 1) % 2
            if ((cfg.show_board == 0) and (self.picked_up == True)):
                self.picked_up_sprite.visible = False
            elif ((cfg.show_board == 1) and (self.picked_up == True)):
                self.picked_up_sprite.visible = True
#            print ("The 'F2' key was pressed, show board changed to ", cfg.show_board)
        elif (symbol == pyglet.window.key.F3):
            ClearSquareMarkers(self)
        elif (symbol == pyglet.window.key.F4):
            ClearAllMarkers(self)
        elif (symbol == pyglet.window.key.J):
            ToggleMarker(self, 0)
        elif (symbol == pyglet.window.key.K):
            ToggleMarker(self, 1)
        elif (symbol == pyglet.window.key.L):
            ToggleMarker(self, 2)
        elif (symbol == pyglet.window.key.SEMICOLON):
            ToggleMarker(self, 3)
        elif symbol == pyglet.window.key.LEFT:
            if self.cube.x > 0:
                self.cube.x -= cfg.img_pix
#            print ("The 'LEFT' key was pressed")
        elif symbol == pyglet.window.key.RIGHT:
            if self.cube.x < self.game_board_x_limit:
                self.cube.x += cfg.img_pix
#            print ("The 'RIGHT' key was pressed")
        elif symbol == pyglet.window.key.UP:
            if self.cube.y < self.game_board_y_limit:
                self.cube.y += cfg.img_pix
#            print ("The 'UP' key was pressed")
        elif symbol == pyglet.window.key.DOWN:
            if self.cube.y > 0:
                self.cube.y -= cfg.img_pix
#            print ("The 'DOWN' key was pressed")


    def on_key_release(self, symbol, modifiers):
        try:
            self.keys_held.pop(self.keys_held.index(symbol))
#            print ("The key was released")
        except:
            pass


    def update(self, dt):

        for i in range(len(self.marble_sprites)):
            if ((self.marble_sprites[i].x == 0) and
                (self.marble_sprites[i].y == 0)):
                self.marble_sprites[i].dx = 0
                self.marble_sprites[i].dy = 0
                self.marble_sprites[i].visible = False
            elif (self.marble_sprites[i].visible == True):

                if ((self.marble_sprites[i].dx != 0) and
                    (self.marble_sprites[i].x >= self.game_board_x_upper_limit)):
                    self.marble_sprites[i].x = self.game_board_x_upper_limit
                    self.marble_sprites[i].dx = 0
                    StopMarble (self, i)
                elif ((self.marble_sprites[i].dx != 0) and
                    (self.marble_sprites[i].x <= (self.game_board_x_lower_limit - cfg.img_pix))):
                    self.marble_sprites[i].x = self.game_board_x_lower_limit - cfg.img_pix
                    self.marble_sprites[i].dx = 0
                    StopMarble (self, i)
                elif (self.marble_sprites[i].dx != 0):
                    if (((self.marble_sprites[i].x % cfg.img_pix) == 0) and
                        ((self.marble_sprites[i].y % cfg.img_pix) == 0)):
                        CheckMarbleChangeDirection (self, i, self.marble_sprites[i].x, self.marble_sprites[i].y)
                    tmp_pix = cfg.tic_pix
                    self.marble_sprites[i].x += tmp_pix * self.marble_sprites[i].dx

                if ((self.marble_sprites[i].dy != 0) and
                    (self.marble_sprites[i].y >= self.game_board_y_upper_limit)):
                    self.marble_sprites[i].y = self.game_board_y_upper_limit
                    self.marble_sprites[i].dy = 0
                    StopMarble (self, i)
                elif ((self.marble_sprites[i].dy != 0) and
                    (self.marble_sprites[i].y <= (self.game_board_y_lower_limit - cfg.img_pix))):
                    self.marble_sprites[i].y = self.game_board_y_lower_limit - cfg.img_pix
                    self.marble_sprites[i].dy = 0
                    StopMarble (self, i)
                elif (self.marble_sprites[i].dy != 0):
                    if (((self.marble_sprites[i].x % cfg.img_pix) == 0) and
                        ((self.marble_sprites[i].y % cfg.img_pix) == 0)):
                        CheckMarbleChangeDirection (self, i, self.marble_sprites[i].x, self.marble_sprites[i].y)
                    tmp_pix = cfg.tic_pix
                    self.marble_sprites[i].y += tmp_pix * self.marble_sprites[i].dy


    def render(self):

        self.clear()

        DrawBoard (self)

        UpdateLabels(self)

        self.fixed_batch.draw()
        self.green_batch.draw()
        self.control_batch.draw()
        self.widget_batch.draw()
        self.fixed_board_batch.draw()
        self.variable_board_batch.draw()
        self.variable_guess_batch.draw()
        self.pointer_bottom_batch.draw()
        self.pointer_top_batch.draw()
        self.marble_batch.draw()
        for i in range(self.history_limit):
            self.color_batch_list[i].draw()
        self.arrow_batch.draw()
        self.marker_batch.draw()
        self.text_batch.draw()

#        self.fps.draw()

        self.flip()


def main():
    window = Window(width=cfg.img_pix*(cfg.game_cols+cfg.control_cols+3), height=cfg.img_pix*(cfg.game_rows+2), caption="Ngfp", resizable=True, fullscreen=False, visible=False)
    pyglet.clock.schedule_interval(window.update, 1/120.0) # update at 60Hz
    pyglet.app.run()


if __name__ == "__main__":
    main()





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
# Copyright (c) Ant <ant@anthive.com>

import os
import pyglet
import sys
import copy
import json
from pathlib import Path
import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gio
from gi.repository import Gtk
from gi.repository import Gdk

import ngfp.config as cfg

from ngfp.board import ClearAndResizeBoard, DrawBoard
from ngfp.marbles import MarbleInMotion


setting = Gio.Settings.new("org.gtk.Settings.FileChooser")
setting.set_boolean("sort-directories-first", True)


class DialogWindow(Gtk.Window):


    def __init__(self):
        Gtk.Window.__init__(self)

        self.set_default_size(500, 200)
        self.set_border_width (30)
#        self.set_title ("DialogWindow")

        self.set_position (Gtk.WindowPosition.CENTER)

        self.box = Gtk.Box (spacing=40, orientation=Gtk.Orientation.VERTICAL)
        self.add(self.box)

        self.label = Gtk.Label("Additional Information")
        self.box.add(self.label)

        button = Gtk.Button("Ok")
        button.connect("clicked", self.on_ok_button_clicked)

        self.box.add(button)

        self.show_all()


    def on_ok_button_clicked(self, widget):

#        print("The OK button was clicked")

        self.destroy()


def DialogWin (title, added):

    info_win = DialogWindow()
    info_win.connect("destroy", Gtk.main_quit)
    info_win.set_title(title)
    info_win.label.set_text(added)
    info_win.show_all()
    Gtk.main()


# print configuration parameters
def print_cfg ():
    print ("C ", cfg.game_cols, cfg.game_rows, cfg.density, cfg.density_fuzz, cfg.class_weights)
    print ("D ", cfg.default_game_cols, cfg.default_game_rows, cfg.default_density, cfg.default_density_fuzz, cfg.default_class_weights)


def LoadConfigOrUseCurrent ():

    if (cfg.full_config_filename.exists() == True):
        with open(cfg.full_config_filename, "r") as fn:
            loaded_config = json.load(fn)
#            print ("LoadConfigOrUseCurrent config loaded : ", loaded_config)
            cfg.default_game_cols = loaded_config[1][0]
            cfg.default_game_rows = loaded_config[1][1]
            cfg.default_density = loaded_config[1][2]
            cfg.default_density_fuzz = loaded_config[1][3]
            cfg.default_class_weights = copy.deepcopy(loaded_config[1][4])
            cfg.game_cols = loaded_config[2][0]
            cfg.game_rows = loaded_config[2][1]
            cfg.density = loaded_config[2][2]
            cfg.density_fuzz = loaded_config[2][3]
            cfg.class_weights = copy.deepcopy(loaded_config[2][4])
            if (cfg.show_board != 2):
                DialogWin (
                    "Ngfp Configuration Loaded",
                    "          from this file for you... " + cfg.config_filename 
                    )

    else:
        # current defaults are set in config.py
        # and we don't want to clobber or reset
        # them unless the user specifically requests it
#        print ("Use Current Parameters")
        if (cfg.show_board != 2):
            DialogWin (
                "Ngfp Configuration file doesn't exist",
                "so the current defaults are being used instead..."
            )

#    print_cfg ()


def SaveConfigToFile ():
#    print_cfg ()
    if (cfg.config_path.exists() != True):
#        print ("Creating : ", str(cfg.config_path))
        cfg.config_path.mkdir(mode=0o700, parents=True, exist_ok=False)
        DialogWin (
            "Ngfp No Configuration Directory Exists",
            "We've created " + str(cfg.config_path) + " for you and put " + cfg.config_filename + " in there."
            )

    with open(cfg.full_config_filename, mode="w") as fileout:
        json.dump([["NGFP_Config\n", 1], [cfg.default_game_cols, cfg.default_game_rows, cfg.default_density, cfg.default_density_fuzz, cfg.default_class_weights],[cfg.game_cols, cfg.game_rows, cfg.density, cfg.density_fuzz, cfg.class_weights]], fileout, indent = 4, separators=(',', ': '))

    DialogWin (
        "Ngfp Configuration Saved",
        "    in this file for you... " + cfg.config_filename
        )


class MyConfigWindow(Gtk.ApplicationWindow):


    def __init__(self, app):
        Gtk.Window.__init__(self, title="Ngfp Configuration", application=app)
        self.set_border_width (30)
        self.set_default_size(600, 600)
        self.set_position (Gtk.WindowPosition.CENTER)

        # adjustments (initial value, min value, max value,
        # step increment - press cursor keys to see!,
        # page increment - click around the handle to see!,
        # page size - not used here)
        adj_width = Gtk.Adjustment(cfg.game_cols, cfg.min_cols, cfg.max_cols, 1, 1, 0)
        adj_height = Gtk.Adjustment(cfg.game_rows, cfg.min_rows, cfg.max_rows, 1, 1, 0)
        adj_density = Gtk.Adjustment(cfg.density, 0, 100, 1, 1, 0)
        adj_density_fuzz = Gtk.Adjustment(cfg.density_fuzz, 0, (100 - cfg.density), 1, 1, 0)

        # a horizontal scale
        self.h1_scale = Gtk.Scale(
            orientation=Gtk.Orientation.HORIZONTAL, adjustment=adj_width)
        # of integers (no digits)
        self.h1_scale.set_digits(0)
        # that can expand horizontally if there is space in the grid (see
        # below)
        self.h1_scale.set_hexpand(False)
        # that is aligned at the top of the space allowed in the grid (see
        # below)
        self.h1_scale.set_valign(Gtk.Align.START)

        # we connect the signal "value-changed" emitted by the scale with the callback
        # function left_h1_scale_moved
        self.h1_scale.connect("value-changed", self.left_h1_scale_moved)

        # 2nd horizontal scale
        self.h2_scale = Gtk.Scale(
            orientation=Gtk.Orientation.HORIZONTAL, adjustment=adj_height)
        # of integers (no digits)
        self.h2_scale.set_digits(0)
        # that can expand horizontally if there is space in the grid (see below)
        self.h2_scale.set_hexpand(False)

        # we connect the signal "value-changed" emitted by the scale with the callback
        # function left_h2_scale_moved
        self.h2_scale.connect("value-changed", self.left_h2_scale_moved)

        # 3rd horizontal scale
        self.h3_scale = Gtk.Scale(
            orientation=Gtk.Orientation.HORIZONTAL, adjustment=adj_density)
        # of integers (no digits)
        self.h3_scale.set_digits(0)
        # that can expand horizontally if there is space in the grid (see below)
        self.h3_scale.set_hexpand(False)

        # we connect the signal "value-changed" emitted by the scale with the callback
        # function left_h3_scale_moved
        self.h3_scale.connect("value-changed", self.left_h3_scale_moved)

        # 4th horizontal scale
        self.h4_scale = Gtk.Scale(
            orientation=Gtk.Orientation.HORIZONTAL, adjustment=adj_density_fuzz)
        # of integers (no digits)
        self.h4_scale.set_digits(0)
        # that can expand horizontally if there is space in the grid (see below)
        self.h4_scale.set_hexpand(False)

        # we connect the signal "value-changed" emitted by the scale with the callback
        # function left_h4_scale_moved
        self.h4_scale.connect("value-changed", self.left_h4_scale_moved)

        # the four labels needed on the left side
        self.l1_label = Gtk.Label()
        self.l1_label.set_text(cfg.property_labels[0])
        self.l2_label = Gtk.Label()
        self.l2_label.set_text(cfg.property_labels[1])
        self.l3_label = Gtk.Label()
        self.l3_label.set_text(cfg.property_labels[2])
        self.l4_label = Gtk.Label()
        self.l4_label.set_text(cfg.property_labels[3])

        # and then the bottom label for the left side
        self.bottom_label_left = Gtk.Label()
        self.bottom_label_left.set_text("Move the scale handles...")
        self.bottom_label_left.set_width_chars (21)

        # a grid to attach the widgets
        self.grid = Gtk.Grid ()
        self.grid.set_row_homogeneous (True)
        self.grid.set_column_spacing (2)
        self.grid.set_row_spacing (20)

        # do the left side first
        self.grid.add (self.h1_scale)
        self.grid.attach_next_to (self.l1_label, self.h1_scale, Gtk.PositionType.BOTTOM, 1, 1)
        self.grid.attach_next_to (self.h2_scale, self.l1_label, Gtk.PositionType.BOTTOM, 1, 1)
        self.grid.attach_next_to (self.l2_label, self.h2_scale, Gtk.PositionType.BOTTOM, 1, 1)
        self.grid.attach_next_to (self.h3_scale, self.l2_label, Gtk.PositionType.BOTTOM, 1, 1)
        self.grid.attach_next_to (self.l3_label, self.h3_scale, Gtk.PositionType.BOTTOM, 1, 1)
        self.grid.attach_next_to (self.h4_scale, self.l3_label, Gtk.PositionType.BOTTOM, 1, 1)
        self.grid.attach_next_to (self.l4_label, self.h4_scale, Gtk.PositionType.BOTTOM, 1, 1)
        self.grid.attach_next_to (self.bottom_label_left, self.l4_label, Gtk.PositionType.BOTTOM, 1, 1)

        class_pack = []
        class_label = []
        class_pic_pack = []
        class_pic_left = []
        class_pic_right = []
        class_adj = []
        self.class_scale = []

        # ok, now the right side
        pic_ind = 0
        j = 0
        col = 1
        for i in range(len(cfg.widget_class_labels)):
            class_pack.append (Gtk.Box (orientation=Gtk.Orientation.VERTICAL))
            class_label.append (Gtk.Label (cfg.widget_class_labels[i]))
            class_pack[i].pack_start (class_label[i], True, False, 0)
            class_label[i].set_width_chars (15)
            class_pic_pack.append (Gtk.Box (spacing=50))
            class_pic_pack[i].set_halign (Gtk.Align.CENTER)
            class_pic_left.append (Gtk.Image.new_from_file(cfg.pic_list[cfg.config_percent_list[pic_ind]]))
            pic_ind += 1
            class_pic_right.append (Gtk.Image.new_from_file(cfg.pic_list[cfg.config_percent_list[pic_ind]]))
            pic_ind += 1
            class_pic_pack[i].pack_start (class_pic_left[i], False, False, 0)
            class_pic_pack[i].pack_start (class_pic_right[i], False, False, 0)
            class_pack[i].pack_start (class_pic_pack[i], True, True, 0)
            class_adj.append (Gtk.Adjustment (cfg.class_weights[i], 1, 100, 1, 1, 0))
            self.class_scale.append (Gtk.Scale (orientation=Gtk.Orientation.HORIZONTAL, adjustment=class_adj[i]))
            self.class_scale[i].set_digits(0)
            class_pack[i].pack_start (self.class_scale[i], True, False, 0)
            self.grid.attach (class_pack[i], col, j, 1, 3)

            # we connect the signal "value-changed" emitted by the scale with the callback
            # function right_scale_moved
            self.class_scale[i].connect("value-changed", self.right_scale_moved)

            if ((i + 1) > 3):
                j = 4
                col = (i + 1) % 4 + 1
            else:
                col += 1

        new_game_button = Gtk.Button.new_with_label("New Game")
        new_game_button.connect("clicked", self.on_new_game_clicked) 
        self.grid.attach (new_game_button, 4, 4, 1, 1)

        save_button = Gtk.Button.new_with_label("Save Config")
        save_button.connect("clicked", self.on_save_clicked)
        self.grid.attach (save_button, 4, 5, 1, 1)

        load_button = Gtk.Button.new_with_label("Load Config")
        load_button.connect("clicked", self.on_load_clicked)
        self.grid.attach (load_button, 4, 6, 1, 1)

        cancel_button = Gtk.Button.new_with_label("Cancel")
        cancel_button.connect("clicked", self.on_cancel_clicked)
        self.grid.attach (cancel_button, 4, 7, 1, 1)

        restore_button = Gtk.Button.new_with_label("Defaults")
        restore_button.connect("clicked", self.on_restore_clicked)
        self.grid.attach (restore_button, 4, 8, 1, 1)

        self.add(self.grid)


    # any signal from the left property scales is signaled to the
    # bottom_label_left text of which is changed and also the
    # respective config parameters
    def left_h1_scale_moved(self, event):
        cfg.game_cols = int(self.h1_scale.get_value())
        self.bottom_label_left.set_text("W " + str(cfg.game_cols) + " H " + str(cfg.game_rows) + " D " + str(cfg.density) + " F " + str(cfg.density_fuzz))

    def left_h2_scale_moved(self, event):
        cfg.game_rows = int(self.h2_scale.get_value())
        self.bottom_label_left.set_text("W " + str(cfg.game_cols) + " H " + str(cfg.game_rows) + " D " + str(cfg.density) + " F " + str(cfg.density_fuzz))

    def left_h3_scale_moved(self, event):
        cfg.density = int(self.h3_scale.get_value())
        self.bottom_label_left.set_text("W " + str(cfg.game_cols) + " H " + str(cfg.game_rows) + " D " + str(cfg.density) + " F " + str(cfg.density_fuzz))

    def left_h4_scale_moved(self, event):
        cfg.density_fuzz = int(self.h4_scale.get_value())
        self.bottom_label_left.set_text("W " + str(cfg.game_cols) + " H " + str(cfg.game_rows) + " D " + str(cfg.density) + " F " + str(cfg.density_fuzz))


    # any signals from the right class scales is signaled to
    # update the config class_weights
    def right_scale_moved(self, event):
        for i in range(len(self.class_scale)):
            new_value = int(self.class_scale[i].get_value())
            if (new_value != cfg.class_weights[i]):
#                print ("Class Weights NV i ", i, new_value)
                cfg.class_weights[i] = new_value


    def on_new_game_clicked(self, widget):
#        print ("New Game")
        cfg.show_board = 2  # reinitialize sprites and lists
        cfg.do_random_board = True
        cfg.new_game_cols = cfg.game_cols
        cfg.new_game_rows = cfg.game_rows
        self.destroy ()
#        print_cfg ()


    def on_save_clicked(self, widget):
#        print ("Save Config")
        SaveConfigToFile ()


    def on_load_clicked(self, widget):
#        print ("Load Config")
        LoadConfigOrUseCurrent ()
        self.h1_scale.set_value(cfg.game_cols)
        self.h2_scale.set_value(cfg.game_rows)
        self.h3_scale.set_value(cfg.density)
        self.h4_scale.set_value(cfg.density_fuzz)
        for i in range(len(cfg.class_weights)):
            self.class_scale[i].set_value (cfg.class_weights[i])


    def on_cancel_clicked(self, widget):
#        print ("Cancel")
        self.destroy ()
#        print_cfg ()


    def on_restore_clicked(self, widget):
#        print ("Restore Defaults")
        cfg.game_cols = cfg.default_game_cols
        cfg.game_rows = cfg.default_game_rows
        cfg.density = cfg.default_density
        cfg.density_fuzz = cfg.default_density_fuzz
        for i in range(len(cfg.class_weights)):
            cfg.class_weights[i] = cfg.default_class_weights[i]
            self.class_scale[i].set_value (cfg.default_class_weights[i])
        self.h1_scale.set_value(cfg.game_cols)
        self.h2_scale.set_value(cfg.game_rows)
        self.h3_scale.set_value(cfg.density)
        self.h4_scale.set_value(cfg.density_fuzz)
        if (cfg.show_board != 2):
            DialogWin (
                "      Ngfp Configuration Parameters Reset      ",
                "                      using default values...               "
            )

#        print_cfg ()


class MyConfigApplication(Gtk.Application):


    def __init__(self):
        Gtk.Application.__init__(self)


    def do_activate(self):
        win = MyConfigWindow(self)
        win.show_all()


    def do_startup(self):
        Gtk.Application.do_startup(self)


def ConfigGame (self):

#    print ("ConfigGame dialog ...")
    app = MyConfigApplication()
    app.run()


def SimpleCheck (self):

#    print ("SimpleCheck")

    for i in range(len(self.board)):
        if (self.board[i][0] != self.board[i][1]):
            return (False)
    return (True)


def ComplexCheck (self):

#    print ("ComplexCheck")

    tmp_board = copy.deepcopy (self.board)

    board_match = True
    for i in range(len(self.white_active_squares)):
        board_out = MarbleInMotion (self, 0, 0, 0, self.white_active_squares[i], False)
        guess_out = MarbleInMotion (self, 1, 0, 0, self.white_active_squares[i], False)

        if (board_out != guess_out):
            board_match = False
            break

    self.board = copy.deepcopy (tmp_board)

    return (board_match)


def CheckBoard (self):

#    print ("CheckBoard dialog ...")

    if (SimpleCheck (self) == True):
        DialogWin (
            "You Won!  Exact Match Found.",
            "You've solved the puzzle.  Congratulations!!!"
            )
    elif (ComplexCheck (self) == True):
        DialogWin (
            "You Won!  Functionally Identical Match Found",
            "You've solved the puzzle.  Congratulations!!!"
            )
    else:
        DialogWin (
            "You Lost.  No Match Found.",
            "           Sorry.  Try again...              "
            )


def add_load_filters(dialog):

    filter_json = Gtk.FileFilter()
    filter_json.set_name("json files")
    filter_json.add_pattern("*.json")
    dialog.add_filter(filter_json)

    filter_gfp = Gtk.FileFilter()
    filter_gfp.set_name("gfp files")
    filter_gfp.add_pattern("*.gfp")
    dialog.add_filter(filter_gfp)


def add_save_filters(dialog):

    filter_json = Gtk.FileFilter()
    filter_json.set_name("json files")
    filter_json.add_pattern("*.json")
    dialog.add_filter(filter_json)


class MyOpenWindow(Gtk.Window):

    def __init__(self):
        Gtk.Window.__init__(self, title="Game File Open", has_focus=False)

        self.set_border_width (20)
        self.set_default_size (300,100)
        self.set_position (Gtk.WindowPosition.MOUSE)

        self.dialog = Gtk.FileChooserDialog("Please choose a game file to open", self,
        Gtk.FileChooserAction.OPEN,
            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
            "Select", Gtk.ResponseType.OK))

        add_load_filters(self.dialog)
        self.dialog.set_border_width (20)
        self.dialog.set_default_size (300,100)
        self.dialog.set_position (Gtk.WindowPosition.MOUSE)

        self.dialog.show_all()
        response = self.dialog.run()
        if response == Gtk.ResponseType.OK:
#            print("Select clicked")
            cfg.dialog_cancelled = False
            cfg.this_fn_to_open = str(self.dialog.get_filename())
        elif response == Gtk.ResponseType.CANCEL:
#            print("Cancel clicked")
            cfg.dialog_cancelled = True

        self.dialog.destroy()
        self.quit(self)

    def quit (self, widget, event=None):
        #return True # prevent closing
        return False # close


class MySaveAsWindow(Gtk.Window):

    def __init__(self):
        Gtk.Window.__init__(self, title="Game File Save As", has_focus=False)

        self.set_border_width (20)
        self.set_default_size (300,100)
        self.set_position (Gtk.WindowPosition.MOUSE)

        self.dialog = Gtk.FileChooserDialog("Save your game as", self,
        Gtk.FileChooserAction.SAVE,
            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
            "Select", Gtk.ResponseType.OK))

        add_save_filters(self.dialog)
        self.dialog.set_border_width (20)
        self.dialog.set_default_size (300,100)
        self.dialog.set_position (Gtk.WindowPosition.MOUSE)

        self.dialog.set_do_overwrite_confirmation(self.dialog)

        if (cfg.this_fn_to_save == None):
            self.dialog.set_current_name(str(cfg.data_path / Path(cfg.suggested_fn)))
        else:
            self.dialog.set_filename(cfg.this_fn_to_save)

        self.dialog.show_all()
        response = self.dialog.run()
        if response == Gtk.ResponseType.OK:
            print("Select clicked")
            cfg.dialog_cancelled = False
            cfg.this_fn_to_save = self.dialog.get_filename()
#            print("Save File selected: " + self.dialog.get_filename())
        elif response == Gtk.ResponseType.CANCEL:
#            print("Cancel clicked")
            cfg.dialog_cancelled = True

        self.dialog.destroy()
        self.quit(self)

    def quit (self, widget, event=None):
        #return True # prevent closing
        return False # close


class TextViewWindow(Gtk.Window):

    def __init__(self):

        Gtk.Window.__init__(self, title="About Ngfp")

        self.top_display = pyglet.canvas.get_display()
        self.top_screen = self.top_display.get_default_screen()
        self.full_screen_width = self.top_screen.width
        self.full_screen_height = self.top_screen.height
        self.set_default_size(self.full_screen_width, (self.full_screen_height -250))

        self.grid = Gtk.Grid()
        self.add(self.grid)

        self.create_textview()


    def create_textview(self):
        scrolledwindow = Gtk.ScrolledWindow()
        scrolledwindow.set_hexpand(True)
        scrolledwindow.set_vexpand(True)
        self.grid.attach(scrolledwindow, 0, 1, 3, 1)

        self.textview = Gtk.TextView()
        self.textbuffer = self.textview.get_buffer()
        self.textbuffer.set_text(
            "\n    Ngfp is running in directory : " + str(Path.cwd()) + "\n"
            + "\n"
            + "      It saves game files to directory : " + str(cfg.data_path) + "\n"
            + "\n"
            + "        Open file name : " + str(cfg.this_fn_to_open) + "\n"
            + "        Save file name : " + str(cfg.this_fn_to_save) + "\n"
            + "\n"
            + "      It keeps configuration settings in directory : " + str(cfg.config_path) + "\n"
            + "        Configuration file name : " + cfg.config_filename + "\n"
            + "\n"
            + "\n"
            + "    For Game Help Press H, F1 or ?\n"
            + "      On a Linux system there should also be a man page.\n"
            + "\n"
            + "    To Quit Playing Game Press Q or ESC\n"
            + "\n"
            + "\n"
            + "    Project Location :"
            + "    https://salsa.debian.org/ant-guest/gfpoken-in-python\n"
            + "\n"
            + "    To leave this window close it..."
            )

        scrolledwindow.add(self.textview)
        self.textview.set_editable(False)
        self.textview.set_cursor_visible(False)


def ShowAbout (self):

#    print_cfg ()
    win = TextViewWindow()
    win.connect("destroy", Gtk.main_quit)
    win.show_all()
    Gtk.main()


class HelpViewWindow(Gtk.Window):

    def __init__(self):

        Gtk.Window.__init__(self, title="Ngfp Help")

        self.top_display = pyglet.canvas.get_display()
        self.top_screen = self.top_display.get_default_screen()
        self.full_screen_width = self.top_screen.width
        self.full_screen_height = self.top_screen.height
        self.set_default_size(self.full_screen_width, (self.full_screen_height -250))

        self.grid = Gtk.Grid()
        self.add(self.grid)

        self.create_textview()


    def create_textview(self):
        scrolledwindow = Gtk.ScrolledWindow()
        scrolledwindow.set_hexpand(True)
        scrolledwindow.set_vexpand(True)
        self.grid.attach(scrolledwindow, 0, 1, 3, 1)

        self.textview = Gtk.TextView()
        self.textbuffer = self.textview.get_buffer()
        self.textbuffer.set_text(
            "\n  Ngfp is a puzzle game.  When first started you will be shown a blank grid on the left and on the right a selection of mirrors.  Under each mirror is the number of how many that need to be placed to solve the puzzle.  You have to use all of the mirrors.\n"
            + "\n"
            + "  To place a mirror you can left mouse click to select it and then click again on the grid where you want to put it down.  To rotate the mirror after it has been placed you right mouse click on it.  To move a mirror left click on it to select it and then left click it and then left click again where you want to place it.\n"
            + "\n"
            + "  To discover where to put the mirrors you left mouse click on any white square that borders the grid and this starts a colored marble rolling through the grid.  If you have already placed mirrors then the marble will be reflected or absorbed in various ways depending upon the type of mirror.  Each colored marble will have a matching colored arrow marking where it started, but not all marbles will have a matching out arrow (some mirrors capture or absorb the marble or it is possible for the game to generate mirrors which can send the marble in a loop which does not exit).\n"
            + "\n"
            + "  You can place up to four markers in a grid location to keep track of what you think may be there.  'J', 'K', 'L' and ';' will put a mark (or turn the mark off if you've already made one) in the grid location your mouse last was over.  F3 will clear the marks from that grid location, F4 will clear all marks (there are no undos for F3 or F4).\n"
            + "\n"
            + "  There are two kinds of solutions to the puzzle.  One is an exact match and the other is a functional match (where all marbles put in from the edges come out where they are supposed to, but the mirrors may not be in the exact same place as the puzzle) - both are considered Wins.\n"
            + "\n"
            + "  If you seem to not be getting any response from the game it is likely that you have pressed F2 or the Show Board Icon or have some other game window open.  If you have pressed F2 you can press F2 again to return to playing the game.  There is no going back from pressing the Show Board Icon.  Or close the other game window.\n"
            + "\n"
            + "  Configuration information can be changed, game checked and games can be loaded and saved via the icons in the green column in the center.\n"
            + "\n"
            + "\n"
            + "    On a Linux system there should be a man page.\n"
            + "\n"
            + "\n"
            + "  Mouse:\n"
            + "\n"
            + "    Left mouse click picks up and drops mirrors."
            + "    Right mouse click rotates mirrors."
            + "\n"
            + "\n"
            + "  Keys:\n"
            + "\n"
            + "    J, K, L and ; are ways to place markers.\n"
            + "    F3 clears markers in one grid location.\n"
            + "    F4 clears all markers.\n"
            + "\n"
            + "\n"
            + "    To Quit Playing Game Press Q or ESC.\n"
            + "\n"
            + "    H, ? or F1 should bring up this help text.\n"
            + "\n"
            + "    F2 is a way to give up and see the puzzle.\n"
            + "\n"
            + "\n"
            + "    To leave this window close it..."
            )

        scrolledwindow.add(self.textview)
        self.textview.set_editable(False)
        self.textview.set_cursor_visible(False)
        self.textview.set_wrap_mode(Gtk.WrapMode.WORD)


def ShowHelp (self):

#    print_cfg ()
    win = HelpViewWindow()
    win.connect("destroy", Gtk.main_quit)
    win.show_all()
    Gtk.main()


def Load_GFPSave_Version_1 (self, lines_in):

    line = lines_in.pop(0)
    line = lines_in.pop(0)
    board_dimensions_strings = line.split()
    cfg.new_game_cols = int(board_dimensions_strings[0])
    cfg.new_game_rows = int(board_dimensions_strings[1])

    # remember the lists are numbered from bottom left in pyglet
    # so board[0][0] is the 1st item of the last line, which is why 
    # we reverse them
    guess_lines = lines_in[:cfg.new_game_rows]
    marker_lines = lines_in[cfg.new_game_rows:cfg.new_game_rows*2]
    board_lines = lines_in[cfg.new_game_rows*2:cfg.new_game_rows*3]

    cfg.new_board = []
    for i in reversed(range(len(board_lines))):
        line_str = board_lines[i]
        for item_ind in line_str.split():
            cfg.new_board.append([int(item_ind), 0])

    board_ind = 0
    for i in reversed(range(len(guess_lines))):
        line_str = guess_lines[i]
        for item_ind in line_str.split():
            cfg.new_board[board_ind][1] = int(item_ind)
            board_ind += 1

    counters_str = lines_in[len(lines_in)-1]
    cfg.new_widget_counts = []
    for i in counters_str.split():
        cfg.new_widget_counts.append(int(i))

    # add a zero for extra widget
    cfg.new_widget_counts.append(0)

    # we're going to have to redraw the board
    # but we aren't a random board
    cfg.show_board = 2
    cfg.do_random_board = False

#    print ("Load_GFPSave_Version_1 -> new variables NC NR NB NWC", cfg.new_game_cols, cfg.new_game_rows, cfg.new_board, cfg.new_widget_counts)


def Load_NGFPSave_Version_1 (self, lines_in):

    cfg.new_game_cols = lines_in[1][0]
    cfg.new_game_rows = lines_in[1][1]

    cfg.new_board = []
    cfg.new_board = copy.deepcopy(lines_in[2])

    cfg.new_widget_counts = []
    cfg.new_widget_counts = copy.deepcopy(lines_in[3])

    # we're going to have to redraw the board
    # but we aren't a random board
    cfg.show_board = 2
    cfg.do_random_board = False

#    print ("Load_NGFPSave_Version_1 -> new variables NC NR NB NWC", cfg.new_game_cols, cfg.new_game_rows, cfg.new_board, cfg.new_widget_counts)


def LoadSavedGameFromFile (self):

#    print ("LoadSavedGameFromFile...")

    cfg.saved_dir = str(Path.cwd())
#    print ("Keep track of current directory : ", cfg.saved_dir)

    # check for saved games directory
    if (cfg.data_path.exists() != True):
        DialogWin (
            "Ngfp Save Game Directory Missing",
            "You haven't created : " + str(cfg.data_path) + " yet..."
            )
        return

    # is there anything in there?
    if (len(os.listdir(path=str(cfg.data_path))) == 0):
        DialogWin (
            "Ngfp Save Game Directory Empty",
            "       There are no saved games yet...       "
            )
        return

#    print ("Changing directory to : ", str(cfg.data_path))
    os.chdir(str(cfg.data_path))
    win = MyOpenWindow()
    win.connect("destroy", Gtk.main_quit)
    win.show_all()
    win.hide()

#    print ("Going back to directory : ", cfg.saved_dir)
    os.chdir(cfg.saved_dir)

    if (cfg.dialog_cancelled == True):
#        print ("Load Dialog Cancelled")
        return

    if (cfg.this_fn_to_open == None):
        print ("LoadSavedGameFromFile...  No file selected...")
        return

    print ("Open File selected : ", cfg.this_fn_to_open)
    if (cfg.this_fn_to_open.endswith(".gfp") == True):
        try:
            with open(cfg.this_fn_to_open) as filein:
                lines_in = filein.readlines()
            Load_GFPSave_Version_1 (self, lines_in)
            cfg.show_board = 2  # reinitialize sprites and lists
            cfg.do_random_board = False
            cfg.this_fn_to_save = cfg.this_fn_to_open
            DrawBoard (self)
        except:
            DialogWin (
                "Ngfp Loading a previously saved game failed",
                "  not sure what isn't working with this file : " + str(cfg.this_fn_to_save)
                )
#            print ("LoadSavedGameFromFile : ", cfg.this_fn_to_open, "didn't load...")
    elif (cfg.this_fn_to_open.endswith(".json") == True):
        try:
            with open(cfg.this_fn_to_open) as filein:
                lines_in = json.load(filein)
            Load_NGFPSave_Version_1 (self, lines_in)
            cfg.show_board = 2  # reinitialize sprites and lists
            cfg.do_random_board = False
            cfg.this_fn_to_save = cfg.this_fn_to_open
            DrawBoard (self)
        except:
            "Ngfp Loading a previously saved game failed",
            "  not sure what isn't working with this file : " + str(cfg.this_fn_to_save
            )
#            print ("LoadSavedGameFromFile : ", cfg.this_fn_to_open, "didn't load...")


def SaveGameToFile (self):

#    print ("SaveGameToFile ...")

    cfg.saved_dir = str(Path.cwd())
#    print ("Keep track of current directory : ", cfg.saved_dir)

    if (cfg.data_path.exists() != True):
#        print ("No save directory exists.  Creating : ", str(cfg.data_path))
        cfg.data_path.mkdir(mode=0o700, parents=True, exist_ok=False)
        DialogWin (
            "Ngfp No Save Game Directory Exists",
            "  We've created " + str(cfg.data_path) + " for you..."
            )

#    print ("Changing directory to : ", str(cfg.data_path))
    os.chdir(str(cfg.data_path))

    win = MySaveAsWindow()
    win.connect("destroy", Gtk.main_quit)
    win.show_all()
    win.hide()

    if (cfg.dialog_cancelled == True):
#        print ("Dialog Cancelled")
#        print ("Going back to directory : ", cfg.saved_dir)
        os.chdir(cfg.saved_dir)
        return

    if (cfg.this_fn_to_save == None):
        print ("SaveGameToFile...  No file selected...")
#        print ("Going back to directory : ", cfg.saved_dir)
        os.chdir(cfg.saved_dir)
        return

#    print ("Saving Game to File : ", cfg.this_fn_to_save)
    with open(cfg.this_fn_to_save, mode="w") as fileout:

        json.dump([["NGFP_Save\n", 1], [cfg.game_cols, cfg.game_rows], self.board, self.widget_pile_list_counts], fileout, indent = 4, separators=(',', ': '))

    cfg.this_fn_to_open = cfg.this_fn_to_save
#    print ("Going back to directory : ", cfg.saved_dir)
    os.chdir(cfg.saved_dir)


def DoDialogControlAction (self, x, x_rec, y, y_rec, win_pos):

#    print ("DoDialogControlAction ", x, x_rec, y, y_rec, win_pos)

    menu_index = self.control_active_squares.index(win_pos)

    # simple menu selection
    if (menu_index == 0):
        # New Game Config Dialog
        ConfigGame (self)
    elif (menu_index == 1):
        # Check Board
        CheckBoard (self)
    elif (menu_index == 2):
        # Flip to Other Board
        cfg.show_board = (cfg.show_board + 1) % 2
    elif (menu_index == 3):
        # Load Saved Game
        LoadSavedGameFromFile (self)
    elif (menu_index == 4):
        # Save Current Game
        SaveGameToFile (self)
    elif (menu_index == 5):
        # About This Game
        ShowAbout (self)
    else:
        pass





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
# Copyright (c) Ant <ant@anthive.com>

import pyglet
import sys
import copy
from time import sleep

import ngfp.config as cfg

from ngfp.history import UpdateAndShowArrow, HideBothArrows, HideOutArrow, HistoryNext, HistoryAndMarbleShift


def MoveLeft (self):

    return (-1)


def MovingLeft (self, dir):

    return (dir == MoveLeft(self))


def MoveRight (self):

    return (1)


def MovingRight (self, dir):

    return (dir == MoveRight(self))


def MoveUp (self):

    return (self.window_cols)


def MovingUp (self, dir):

    return (dir == MoveUp(self))


def MoveDown (self):

    return (-self.window_cols)


def MovingDown (self, dir):

    return (dir == MoveDown(self))


def ReverseDir (self, dir):
    return (-dir)


def ChangeBoard (self, which_board, board_index, widget):

#    print ("ChangeBoard which_board ", which_board, "board_index ", board_index, " widget ", widget)
    self.board[board_index][which_board] = self.widget_next_widget[widget]
    if (which_board == 0):
        self.board_sprites[board_index].image = self.spr_mv_list[self.widget_next_widget[widget]][1]
    elif (which_board == 1):
        self.guess_sprites[board_index].image = self.spr_mv_list[self.widget_next_widget[widget]][1]
    else:
        pass


def MovingWidget (self, which_board, cur_pos, board_index, widget, dir):

#    print ("MovingWidget which_board ", which_board, "cur_pos ", cur_pos, " board_index ", board_index, 
#        " widget ", widget, " direction", dir)
    try:
        new_board_index = self.board_to_window_index.index(cur_pos + dir)
#        print ("    new_board_index ", new_board_index)
        if (((cur_pos + dir) in self.guess_active_squares) and 
           (self.board[new_board_index][which_board] == 0)):
#            print ("    Found a space in square ", new_board_index)
            if (which_board == 0):
                self.board_sprites[board_index].image = self.game_bg_image
                self.board_sprites[board_index].visible = False
                self.board[board_index][0] = 0
                self.board_sprites[new_board_index].image = self.spr_mv_list[widget][1]
                self.board_sprites[new_board_index].visible = True
                self.board[new_board_index][0] = widget
            elif (which_board == 1):
                self.guess_sprites[board_index].image = self.game_bg_image
                self.guess_sprites[board_index].visible = False
                self.board[board_index][1] = 0
                self.guess_sprites[new_board_index].image = self.spr_mv_list[widget][1]
                self.guess_sprites[new_board_index].visible = True
                self.board[new_board_index][1] = widget
            else:
                pass
        else:
#            print ("    No space found, didn't move widget")
            pass
    except:
        pass


def MirrorMagic (self, which_board, cur_pos, cur_dir):

    board_index = self.board_to_window_index.index(cur_pos)
    widget = self.board[board_index][which_board]
    if (which_board == 1):
#        print ("MirrorMagic board ", which_board," board index ", board_index, " widget ", widget)
        pass
    if (widget == 0):   # nothing to do here move along...
#        print ("  MirrorMagic nothing to do here...")
        return (cur_dir)
    elif (widget == 1):   # simple mirrors: left: \
        if (MovingLeft (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (MoveRight (self))
    elif (widget == 2):   # simple mirrors: right: /
        if (MovingLeft (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveRight (self))
        else:
            return (MoveLeft (self))
    elif (widget == 3):   # simple flipping mirrors: left: \
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingLeft (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (MoveRight (self))
    elif (widget == 4):   # simple flipping mirrors: right: / 
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingLeft (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveRight (self))
        else:
            return (MoveLeft (self))
    elif (widget == 5):   # quad flipping mirrors: left: \
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingLeft (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (MoveRight (self))
    elif ((widget == 6) or (widget == 8)):   # flipping mirrors: bounce: o
        ChangeBoard (self, which_board, board_index, widget)
        return (ReverseDir (self, cur_dir))
    elif (widget == 7):   # flipping mirrors: right: /
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingLeft (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveRight (self))
        else:
            return (MoveLeft (self))
    elif (widget == 9):  # box and sink: box: bounce: o  (reflect all)
        return (ReverseDir (self, cur_dir))
    elif (widget == 10):  # box and sink: sink: grab: x  (absorb all)
        return (None)
    elif (widget == 11):  # axial mirrors: simple vertical: |
        if (MovingLeft (self, cur_dir)):
            return (MoveRight (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (cur_dir)
    elif (widget == 12):  # axial mirrors: simple horizontal: -
        if (MovingUp (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingDown (self, cur_dir)):
            return (MoveUp (self))
        else:
            return (cur_dir)
    elif (widget == 13):   # axial mirrors: flipping vertical: ||
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingLeft (self, cur_dir)):
            return (ReverseDir (self, cur_dir))
        elif (MovingRight (self, cur_dir)):
            return (ReverseDir (self, cur_dir))
        else:
            return (cur_dir)
    elif (widget == 14):   # axial mirrors: flipping horizontal: =
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingUp (self, cur_dir)):
            return (ReverseDir (self, cur_dir))
        elif (MovingDown (self, cur_dir)):
            return (ReverseDir (self, cur_dir))
        else:
            return (cur_dir)
    elif (widget == 15):  # rotators simple counterclockwise: left: \\
        if (MovingLeft (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveRight (self))
        else:
            return (MoveLeft (self))
    elif (widget == 16):  # rotators simple clockwise: right: //
        if (MovingLeft (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (MoveRight (self))
    elif (widget == 17):  # rotators flipper clockwise: left: []
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingLeft (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveRight (self))
        else:
            return (MoveLeft (self))
    elif (widget == 18):  # rotators flipper counterclockwise: right: ][
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingLeft (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (MoveRight (self))
    elif (widget == 19):   # 1-way mirrors: left: lower reflects: \<-
        if (MovingRight (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (cur_dir)
    elif (widget == 20):   # 1-way mirrors: left: upper reflects: ->\
        if (MovingLeft (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingDown (self, cur_dir)):
            return (MoveRight (self))
        else:
            return (cur_dir)
    elif (widget == 21):   # 1-way mirrors: right: lower reflects: ->/
        if (MovingLeft (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveRight (self))
        else:
            return (cur_dir)
    elif (widget == 22):   # 1-way mirrors: right: upper reflects: /<-
        if (MovingRight (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingDown (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (cur_dir)
    elif (widget == 23):  # flipping 1-way mirrors: left: lower reflects: rotates clockwise: \\\<-
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingRight (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (cur_dir)
    elif (widget == 24):  # flipping 1-way mirrors: right: upper reflects: rotates clockwise: ///<-
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingRight (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingDown (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (cur_dir)
    elif (widget == 25):  # flipping 1-way mirrors: left: upper reflects: rotates clockwise: ->\\\
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingLeft (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingDown (self, cur_dir)):
            return (MoveRight (self))
        else:
            return (cur_dir)
    elif (widget == 26):  # flipping 1-way mirrors: right: lower reflects: rotates clockwise: ->///
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingLeft (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveRight (self))
        else:
            return (cur_dir)
    elif (widget == 27): # flipping 1-way mirrors: left: upper reflects: rotates counterclockwise: ->\\\
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingLeft (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingDown (self, cur_dir)):
            return (MoveRight (self))
        else:
            return (cur_dir)
    elif (widget == 28): # flipping 1-way mirrors: right: upper reflects: rotates counterclockwise: ///<-
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingRight (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingDown (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (cur_dir)
    elif (widget == 29): # flipping 1-way mirrors: left: lower reflects: rotates counterclockwise: \\\<-
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingRight (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (cur_dir)
    elif (widget == 30): # flipping 1-way mirrors: right: lower reflects: rotates counterclockwise: ->///
        ChangeBoard (self, which_board, board_index, widget)
        if (MovingLeft (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveRight (self))
        else:
            return (cur_dir)
    elif (widget == 31): # moving mirror: left: -->\X\--> ---->\X\
        MovingWidget (self, which_board, cur_pos, board_index, widget, cur_dir)
        if (MovingLeft (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveLeft (self))
        else:
            return (MoveRight (self))
    elif (widget == 32): # moving mirror: right: <--/X/<-- /X/<----
        MovingWidget (self, which_board, cur_pos, board_index, widget, cur_dir)
        if (MovingLeft (self, cur_dir)):
            return (MoveDown (self))
        elif (MovingRight (self, cur_dir)):
            return (MoveUp (self))
        elif (MovingUp (self, cur_dir)):
            return (MoveRight (self))
        else:
            return (MoveLeft (self))
    else:
        print ("  MirrorMagic fell through with widget (there shouldn't be any left)???", widget, " widget_next_widget ", self.widget_next_widget[widget])
        return (cur_dir)


def ShowWhiteInArrow (self, win_pos, visible):

    win_pos_index = self.white_active_squares.index(win_pos)
    SWI_xy_coordinates = copy.deepcopy(self.white_active_squares_position[win_pos_index])
#    print ("SW In Arrow XY coordinates", SWI_xy_coordinates)
    rotate = 0.0
    self.kept_start_pos = win_pos
    self.kept_start_pos_x = SWI_xy_coordinates[0]
    self.kept_start_pos_y = SWI_xy_coordinates[1]
    if (win_pos in self.dir_left):             # if we've clicked on the left side it
        self.start_direction = MoveRight(self) # means we're going right
        self.kept_start_dx = 1
        self.kept_start_dy = 0
        self.kept_start_pos_x += cfg.img_pix - cfg.tic_pix
        SWI_xy_coordinates[0] += cfg.half_img_pix
        image = self.arrow_images[1]
    elif (win_pos in self.dir_right):          # if we've clicked on the right side it
        self.start_direction = MoveLeft(self)  # means we're going left
        self.kept_start_dx = -1
        self.kept_start_dy = 0
        self.kept_start_pos_x -= cfg.tic_pix
        image = self.arrow_images[0]
    elif (win_pos in self.dir_up):             #  etc.
        self.start_direction = MoveDown(self)
        self.kept_start_dx = 0
        self.kept_start_dy = -1
        self.kept_start_pos_y -= cfg.img_pix - cfg.tic_pix
        image = self.arrow_images[3]
        rotate = 90.0
    else:
        self.start_direction = MoveUp(self)
        self.kept_start_dx = 0
        self.kept_start_dy = 1
        self.kept_start_pos_y += cfg.img_pix - cfg.tic_pix
        SWI_xy_coordinates[1] += cfg.half_img_pix
        image = self.arrow_images[2]
        rotate = 90.0
    self.kept_start_direction = self.start_direction
    UpdateAndShowArrow(self, image, 0, SWI_xy_coordinates, rotate, visible)


def ShowWhiteOutArrow (self, win_pos, visible):

    if (self.stop_direction == None):
#        print ("SW Out Arrow __No__ Arrow Out")
        pass
    else:
        win_pos_index = self.white_active_squares.index(win_pos)
        SWO_xy_coordinates = copy.deepcopy(self.white_active_squares_position[win_pos_index])
#        print ("SW Out Arrow XY coordinates", SWO_xy_coordinates)
        rotate = 0.0
        if (win_pos in self.dir_left):             # if on the left side it
            self.stop_direction = MoveLeft(self)   # means we're going left
            image = self.arrow_images[0]
        elif (win_pos in self.dir_right):          # if on the right side it
            self.stop_direction = MoveRight(self)  # means we're going right
            SWO_xy_coordinates[0] += cfg.half_img_pix
            image = self.arrow_images[1]
        elif (win_pos in self.dir_up):             #  etc.
            self.stop_direction = MoveUp(self)
            SWO_xy_coordinates[1] += cfg.half_img_pix
            image = self.arrow_images[2]
            rotate = 90.0
        else:
            self.stop_direction = MoveDown(self)
            image = self.arrow_images[3]
            rotate = 90.0
        UpdateAndShowArrow(self, image, 1, SWO_xy_coordinates, rotate, visible)


def StartMarble (self, win_pos, visible):

    self.marble_current_pos = self.kept_start_pos + self.kept_start_direction
    self.marble_current_direction = self.kept_start_direction

#    print ("Start Marble CP, CD ", self.marble_current_pos, self.marble_current_direction)

    self.marble_sprites[0].x = self.kept_start_pos_x
    self.marble_sprites[0].y = self.kept_start_pos_y
    self.marble_sprites[0].dx = self.kept_start_dx
    self.marble_sprites[0].dy = self.kept_start_dy
    self.marble_sprites[0].visible = visible
    for i in range (len (self.marble_sprites) - 1):
        self.marble_sprites[i+1].x = 0
        self.marble_sprites[i+1].y = 0
        self.marble_sprites[i+1].dx = 0
        self.marble_sprites[i+1].dy = 0
        self.marble_sprites[i+1].visible = False

#    print ("Start Marble CP, CD,  x,y with dx,dy ", 
#        self.marble_current_pos,
#        self.marble_current_direction,
#        self.marble_sprites[0].x,
#        self.marble_sprites[0].y,
#        self.marble_sprites[0].dx,
#        self.marble_sprites[0].dy)


def StopMarble (self, win_pos):

    pass


def MarbleInMotion (self, which_board, x_rec, y_rec, win_pos, visible):

    HideBothArrows (self)
    start_pos = win_pos
    self.start_direction = None
    self.stop_direction = None
    self.cube.x = 0
    self.cube.y = 0
    self.cube.visible = False
    self.pointer_top_batch.draw()
    self.variable_guess_batch.draw()
    self.flip()
    ShowWhiteInArrow (self, win_pos, visible)
    for i in range(self.history_limit):
        self.color_batch_list[i].draw()
    self.marble_tick_count = 0
    StartMarble(self, win_pos, visible)
    self.marble_batch.draw()
    self.arrow_batch.draw()
    self.flip()
    self.cube.visible = True
#    print ("Starting to move game marble from ", start_pos, "in direction ", self.start_direction)

    tick_count = 0  #  we better have some limit just in case of long loops
    current_pos = start_pos + self.start_direction
    current_direction = self.start_direction
#    print ("First move game marble from ", current_pos, "in direction ", current_direction)
    while (
        (current_pos in self.guess_active_squares) and 
        (tick_count < self.tick_limit) and 
        (current_pos != None)):
        tick_count += 1
        new_dir = MirrorMagic (self, which_board, current_pos, current_direction)
        if (new_dir != None):
#            if (current_pos in self.guess_active_squares):
#                cube_xy = self.guess_active_squares_position[self.guess_active_squares.index(current_pos)]
#                print ("   Cube xy ", cube_xy)
#                self.cube.visible = True
#                self.cube.x = cube_xy[0]
#                self.cube.y = cube_xy[1]
#                self.pointer_top_batch.draw()
#                self.flip()
#            print ("Moved: game marble Tick, CP, CD ", tick_count, current_pos, current_direction)
#            sleep (0.08)
            current_pos += new_dir
            current_direction = new_dir
            self.stop_direction = current_direction
        else:
#            print ("Didn't Move: game marble Tick, CP, CD ", tick_count, current_pos, current_direction)
            self.stop_direction = None
            break

#    print ("Last moved game marble at ", current_pos, "in direction ", current_direction)
    if (tick_count >= self.tick_limit):
#        print ("Exceeded game marble tick_count of ", self.tick_limit)
        return_value = -1
    elif (self.stop_direction == None):
#        print ("No game marble Arrow Out...")
        HideOutArrow (self)
        return_value = None
    else:
        ShowWhiteOutArrow (self, current_pos, visible)
        return_value  = current_pos
    HistoryNext (self)
    HistoryAndMarbleShift (self)
    return (return_value)


def DoLeftClickWhiteAction (self, x, x_rec, y, y_rec, win_pos):

    value = MarbleInMotion (self, 0, x_rec, y_rec, win_pos, True)
#    print ("MIM 0 T ret_val ", value)


def CheckMarbleChangeDirection (self, sprite_index, x, y):

#    print ("Check Marble Change Direction at x,y ", x, y)
    self.marble_tick_count += 1
    if (self.marble_current_pos not in self.guess_active_squares): 
#        print ("Check Marble Tick, CP, CD Not in guess active squares ", self.marble_tick_count, self.marble_current_pos, self.marble_current_direction)
        pass
    else:
#        print ("Check Marble Tick, CP, CD ", self.marble_tick_count, self.marble_current_pos, self.marble_current_direction)
        old_dir = self.marble_current_direction
        new_dir = MirrorMagic (self, 1, self.marble_current_pos, self.marble_current_direction)
        if (new_dir != None):
            self.marble_current_pos += new_dir
            self.marble_current_direction = new_dir
            self.marble_stop_direction = self.marble_current_direction
            if (old_dir == new_dir):
#               print ("Check Marble didn't change direction... ")
                pass
            elif (old_dir == -new_dir):
#                print ("Check Marble reversed direction... ")
                self.marble_sprites[sprite_index].dx *= -1
                self.marble_sprites[sprite_index].x +=  cfg.tic_pix * self.marble_sprites[sprite_index].dx
                self.marble_sprites[sprite_index].dy *= -1
                self.marble_sprites[sprite_index].y +=  cfg.tic_pix * self.marble_sprites[sprite_index].dy
            elif ((MovingLeft (self, old_dir) or MovingRight (self, old_dir)) and
                  (MovingUp (self, new_dir) or MovingDown (self, new_dir))):
#                print ("Check Marble reflected Up or Down... ")
                self.marble_sprites[sprite_index].dx = 0
                self.marble_sprites[sprite_index].dy = new_dir//abs(new_dir)
                self.marble_sprites[sprite_index].y +=  cfg.tic_pix * self.marble_sprites[sprite_index].dy
            elif ((MovingUp (self, old_dir) or MovingDown (self, old_dir)) and
                  (MovingLeft (self, new_dir) or MovingRight (self, new_dir))):
#                print ("Check Marble reflected Left or Right... ")
                self.marble_sprites[sprite_index].dy = 0
                self.marble_sprites[sprite_index].dx = new_dir//abs(new_dir)
                self.marble_sprites[sprite_index].x +=  cfg.tic_pix * self.marble_sprites[sprite_index].dx
            else:
                pass
        else:
#            print ("Didn't Move: Check Marble Tick, CP, CD ", self.marble_tick_count, self.marble_current_pos, self.marble_current_direction)
            self.marble_sprites[sprite_index].dx = 0
            self.marble_sprites[sprite_index].dy = 0
            self.marble_stop_direction = None
            self.marble_tick_count = self.tick_limit

    if (self.marble_tick_count > self.tick_limit):
#        print ("Check Marble Exceeded tick_count of ", self.tick_limit)
        pass





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
# Copyright (c) Ant <ant@anthive.com>

import pyglet
import sys

import ngfp.config as cfg


def HideInArrow (self):

#    print ("HideInArrow ", self.arrow_index)
    self.arrow_history_sprites[self.arrow_index][0].visible = False
    self.history_color_sprites[self.arrow_index][0].visible = False


def HideOutArrow (self):

#    print ("HideOutArrow ", self.arrow_index)
    self.arrow_history_sprites[self.arrow_index][1].visible = False
    self.history_color_sprites[self.arrow_index][1].visible = False


def HideBothArrows (self):

#    print ("HideBothArrows ", self.arrow_index)
    HideInArrow (self)
    HideOutArrow (self)


def UpdateAndShowArrow (self, image, spot, xy_coord, rotate, visible):

#    print ("Update Arrow  arrow_index ", self.arrow_index,
#        " spot, ", spot ," coord", xy_coord)

    self.arrow_history_sprites[self.arrow_index][spot].x = xy_coord[0]
    self.arrow_history_sprites[self.arrow_index][spot].y = xy_coord[1]
    self.arrow_history_sprites[self.arrow_index][spot].image = image
    self.arrow_history_sprites[self.arrow_index][spot].visible = visible

    self.history_color_sprites[self.arrow_index][spot].rotation = rotate
    self.history_color_sprites[self.arrow_index][spot].x = xy_coord[0]
    if (rotate == 0.0):
        self.history_color_sprites[self.arrow_index][spot].y = xy_coord[1]
    else:
        self.history_color_sprites[self.arrow_index][spot].y = xy_coord[1] + cfg.half_img_pix
    self.history_color_sprites[self.arrow_index][spot].visible = visible


def HistoryNext (self):

    self.arrow_index = (self.arrow_index + 1) % self.history_limit


def HistoryAndMarbleShift (self):

    # we need to rotate both the arrows and marbles together
    self.color_batch_list.append(self.color_batch_list.pop(0))
    self.marble_sprites.append(self.marble_sprites.pop(0))





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
# Copyright (c) Ant <ant@anthive.com>

import copy
import json
from pathlib import Path, PurePath
import os


# the path to the images
#print (os.path.basename(__file__))
png_path = os.path.dirname(__file__) + "/png/"
#print (png_path)


# the basic game unit for images and moving is
img_pix = 64


half_img_pix = img_pix//2


# animation pixels moved (it must be a factor of img_pix otherwise
#   the marble won't match with the grid of coordinates - i.e. there's
#   no wiggle room in the collision detection)...
tic_pix = img_pix // 2


# some space for widget piles at the right
control_cols = 5


# dialog cancelled operation
dialog_cancelled = False


#   save game location and initial file
# you can always save/load other names, 
# this is just a suggestion...
suggested_fn = "save.json"
this_fn_to_open = None
this_fn_to_save = None

home = Path.home()
saved_dir = None


# save file directory
if (os.name == "posix"):
    home = Path.home()
    data_path = home / Path(".local/share/ngfp")
else:
    print ("  Ngfp doesn't know where to set data_path for OS : ", os.name)
    print ("This is where a user would save their games.")


# configure location and file
config_filename = "config_ngfp.json"
if (os.name == "posix"):
    home = Path.home()
    config_path = home / Path(".config/ngfp")
    full_config_filename = config_path / config_filename
else:
    print ("  Ngfp doesn't know where to set config_path for OS : ", os.name)
    print ("This is where the game saves configuration parameters.")


# when loading a game from a file we put stuff in
# these spots so that DrawBoard can use them.
# changing dimensions in the config dialog also uses
# these.
new_game_cols = None
new_game_rows = None
new_board = None
new_widget_counts = None


# which board to show, a toggle between 0, 1  when Key F1 is pressed
#    but we start at 2 for a new board and it's random to start...
# 0 - puzzle to solve
# 1 - guesses placed
# 2 - blank background
#
show_board = 2
do_random_board = True


# current, default and changed parameters

min_cols = 1      # this actually works
min_rows = 6      # for the moment
max_cols = 22     # on 1920 x 1080
max_rows = 13     # on 1920 x 1080

game_cols = 6     # width
game_rows = 6     # height
density = 25      # percent of the board filled up modified by density_fuzz
density_fuzz = 10
class_weights = [100, 75, 5, 5, 5, 15, 5]

default_game_cols = 6     # width
default_game_rows = 6     # height
default_density = 25      # percent of the board filled up modified by density_fuzz
default_density_fuzz = 10
default_class_weights = [100, 75, 5, 5, 5, 15, 5]


# need to keep track of the current square
square = None


# labels for configuration dialog

property_labels = [
    "Width",
    "Height",
    "Density",
    "Fuzz"
    ]

widget_class_labels = [
    "Simple mirrors",
    "Flipping mirrors",
    "Box and sink",
    "Axial mirrors",
    "Rotators",
    "One-way mirrors",
    "PURE EVIL"
    ]

pic_list = [
    png_path + "mirrors/00_bg.png",           # background
    png_path + "mirrors/01_normal.png",       # simple mirrors: left: \
    png_path + "mirrors/02_normal.png",       # simple mirrors: right: /
    png_path + "mirrors/03_flip2.png",        # simple flipping mirrors: left: \
    png_path + "mirrors/04_flip2.png",        # simple flipping mirrors: right: /
    png_path + "mirrors/05_flip4.png",        # quad flipping mirrors: left: \
    png_path + "mirrors/06_flip4.png",        # quad flipping mirrors: bounce: o
    png_path + "mirrors/07_flip4.png",        # quad flipping mirrors: right: /
    png_path + "mirrors/08_flip4.png",        # quad flipping mirrors: bounce: o
    png_path + "mirrors/09_block.png",        # box and sink: box: bounce: o  (reflect all)
    png_path + "mirrors/10_sink.png",         # box and sink: sink: grab: x  (absorb all)
    png_path + "mirrors/11_axial.png",        # axial mirrors: simple vertical: |
    png_path + "mirrors/12_axial.png",        # axial mirrors: simple horizontal: -
    png_path + "mirrors/13_axial2.png",       # axial mirrors: flipping vertical: ||
    png_path + "mirrors/14_axial2.png",       # axial mirrors: flipping horizontal: =
    png_path + "mirrors/15_rotator.png",      # rotators simple counterclockwise: left: \\
    png_path + "mirrors/16_rotator.png",      # rotators simple clockwise: right: //
    png_path + "mirrors/17_rotator2.png",     # rotators flipper clockwise: left: []
    png_path + "mirrors/18_rotator2.png",     # rotators flipper counterclockwise: right: ][
    png_path + "mirrors/19_half.png",         # 1-way mirrors: left: lower reflects: \<-
    png_path + "mirrors/20_half.png",         # 1-way mirrors: left: upper reflects: ->\
    png_path + "mirrors/21_half.png",         # 1-way mirrors: right: lower reflects: ->/
    png_path + "mirrors/22_half.png",         # 1-way mirrors: right: upper reflects: /<-
    png_path + "mirrors/23_half4.png",        # flipping 1-way mirrors: left: lower reflects: rotates clockwise: \\\<-
    png_path + "mirrors/24_half4.png",        # flipping 1-way mirrors: right: upper reflects: rotates clockwise: ///<-
    png_path + "mirrors/25_half4.png",        # flipping 1-way mirrors: left: upper reflects: rotates clockwise: ->\\\
    png_path + "mirrors/26_half4.png",        # flipping 1-way mirrors: right: lower reflects: rotates clockwise: ->///
    png_path + "mirrors/27_half4.png",        # flipping 1-way mirrors: left: upper reflects: rotates counterclockwise: ->\\\
    png_path + "mirrors/28_half4.png",        # flipping 1-way mirrors: right: upper reflects: rotates counterclockwise: ///<-
    png_path + "mirrors/29_half4.png",        # flipping 1-way mirrors: left: lower reflects: rotates counterclockwise: \\\<-
    png_path + "mirrors/30_half4.png",        # flipping 1-way mirrors: right: lower reflects: rotates counterclockwise: ->///
    png_path + "mirrors/31_move.png",         # moving mirror: left: -->\X\--> ---->\X\
    png_path + "mirrors/32_move.png",         # moving mirror: right: <--/X/<-- /X/<----
    png_path + "mirrors/33_bg.png"            # background
]

# this indexes the above picture list so we know which ones are
# used for labels on the configure screen
config_percent_list = [1,2,3,5,9,10,11,14,15,18,19,22,23,31]





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
# Copyright (c) Ant <ant@anthive.com>

import pyglet
import copy
from time import sleep

import ngfp.config as cfg

from ngfp.background import DrawBordersAndBackgrounds
from ngfp.labels import AddLabels
from ngfp.randboard import InitRandomBoardItems


def ClearSquareMarkers (self):

#    print ("ClearSquareMarkers mouse_win_pos : ", self.mouse_win_pos)
    value = self.board_to_window_index.index(self.mouse_win_pos)
    how_many_markers = len(self.marker_images)
    for i in range(how_many_markers):
        self.board[value][i+2] = False
        self.marker_sprites[(value*how_many_markers)+i].visible = False


def ClearAllMarkers (self):

#    print ("ClearAllMarkers")
    value = len(self.board)
    how_many_markers = len(self.marker_images)
    for i in range(value):
        for j in range(how_many_markers):
            self.board[i][j+2] = False
            self.marker_sprites[(i*how_many_markers)+j].visible = False


def ToggleMarker (self, mark):

#    print ("ToggleMarker mark, mouse_win_pos : ", mark, self.mouse_win_pos)
    value = self.board_to_window_index.index(self.mouse_win_pos)
    how_many_markers = len(self.marker_images)
    self.board[value][mark+2] = not(self.board[value][mark+2])
    self.marker_sprites[(value*how_many_markers)+mark].visible = not(self.marker_sprites[(value*how_many_markers)+mark].visible)


def ClearAndResizeBoard (self):


    # ok, let's see...
    self.set_visible(True)

    # use the current cfg values
    self.density = cfg.density
    self.density_fuzz = cfg.density_fuzz
    self.class_weights = cfg.class_weights

    # don't forget these...
    self.board_squares = cfg.game_rows*cfg.game_cols
    self.window_rows = (cfg.game_rows+2)
    self.window_cols = (cfg.game_cols+cfg.control_cols+3)
    self.window_squares = self.window_rows*self.window_cols

    # adjust the main window size to fit

    self.screen_width = ((cfg.game_cols+cfg.control_cols+3) * cfg.img_pix)
    self.screen_height = ((cfg.game_rows+2) * cfg.img_pix)

    self.game_x = (self.full_screen_width//2) - (self.screen_width//2)
    self.game_y = (self.full_screen_height//2) - ((self.screen_height//2))

    self.set_location(self.game_x, self.game_y)
    self.set_size(self.screen_width, self.screen_height)

    # ok, let's see...
#   self.set_visible(True)


    # delete old board and set up new one, but
    #   DrawBoard really gets rid of all the various 
    #   lists,sprites and indexes
    try:
        del self.board
    except AttributeError:
        pass
    self.board = []

    self.board = [[0 for i in range(6)] for j in range(self.board_squares)]

    try:
        del self.widget_pile_list_counts
    except AttributeError:
        pass

    self.widget_pile_list_counts = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]

    # move the gcube
    try:
        self.gcube.x = cfg.img_pix * (cfg.game_cols+1)
    except:
        pass


def DrawBoard (self):

    if (cfg.show_board == 2):

        if (len(self.fixed_sprites) != 0):
            for j in range(len(self.fixed_sprites)):
                #self.fixed_sprites[j].visible = False
                self.fixed_sprites[j].delete()
            del self.fixed_sprites
            self.fixed_sprites = []

        if (len(self.fixed_board_sprites) != 0):
            for j in range(len(self.fixed_board_sprites)):
                #self.fixed_board_sprites[j].visible = False
                self.fixed_board_sprites[j].delete()
            del self.fixed_board_sprites
            self.fixed_board_sprites = []

        if (len(self.board_sprites) != 0):
            for j in range(len(self.board_sprites)):
                #self.board_sprites[j].visible = False
                self.board_sprites[j].delete()
            del self.board_sprites
            self.board_sprites = []

        if (len(self.green_sprites) != 0):
            for j in range(len(self.green_sprites)):
                #self.green_sprites[j].visible = False
                self.green_sprites[j].delete()
            del self.green_sprites
            self.green_sprites = []

        if (len(self.white_active_squares) != 0):
            del self.white_active_squares
            self.white_active_squares = []

        if (len(self.white_active_squares_position) != 0):
            del self.white_active_squares_position
            self.white_active_squares_position = []

        if (len(self.dir_left) != 0):
            del self.dir_left
            self.dir_left = []

        if (len(self.dir_right) != 0):
            del self.dir_right
            self.dir_right = []

        if (len(self.dir_up) != 0):
            del self.dir_up
            self.dir_up = []

        if (len(self.dir_down) != 0):
            del self.dir_down
            self.dir_down = []

        if (len(self.guess_sprites) != 0):
            for j in range(len(self.guess_sprites)):
                #self.guess_sprites[j].visible = False
                self.guess_sprites[j].delete()
            del self.guess_sprites
            self.guess_sprites = []

        if (len(self.guess_active_squares) != 0):
            del self.guess_active_squares
            self.guess_active_squares = []

        if (len(self.guess_active_squares_position) != 0):
            del self.guess_active_squares_position
            self.guess_active_squares_position = []

        if (len(self.board_to_window_index) != 0):
            del self.board_to_window_index
            self.board_to_window_index = []

        if (len(self.control_sprites) != 0):
            for j in range(len(self.control_sprites)):
                #self.control_sprites[j].visible = False
                self.control_sprites[j].delete()
            del self.control_sprites
            self.control_sprites = []

        if (len(self.control_active_squares) != 0):
            del self.control_active_squares
            self.control_active_squares = []

        if (len(self.control_active_squares_position) != 0):
            del self.control_active_squares_position
            self.control_active_squares_position = []

        if (len(self.marker_sprites) != 0):
            for j in range(len(self.marker_sprites)):
                #self.marker_sprites[j].visible = False
                self.marker_sprites[j].delete()
            del self.marker_sprites
            self.marker_sprites = []

        if (len(self.widget_active_squares) != 0):
            del self.widget_active_squares
            self.widget_active_squares = []

        if (len(self.widget_active_squares_position) != 0):
            del self.widget_active_squares_position
            self.widget_active_squares_position = []

        if (len(self.arrow_history_sprites) != 0):
            for j in range(len(self.arrow_history_sprites)):
                self.arrow_history_sprites[j][0].visible = False
                self.arrow_history_sprites[j][1].visible = False

        if (len(self.history_color_sprites) != 0):
            for j in range(len(self.history_color_sprites)):
                self.history_color_sprites[j][0].visible = False
                self.history_color_sprites[j][1].visible = False

        if (len(self.marble_sprites) != 0):
            for j in range(len(self.marble_sprites)):
                self.marble_sprites[j].visible = False

        if (cfg.do_random_board == True):
#            print ("Draw Random Board")

            ClearAndResizeBoard (self)
            InitRandomBoardItems (self)
            DrawBordersAndBackgrounds (self)
            cfg.do_random_board = False
#            print ("DrawR self.wpl ", self.widget_pile_list)
#            print ("DrawR self.wplc ", self.widget_pile_list_counts)
        else:
            # it's a newly loaded board
#            print ("Draw Loaded Board")

            cfg.game_cols = cfg.new_game_cols
            cfg.game_rows = cfg.new_game_rows
            # we use ClearAndResizeBoard to put the window on
            # the screen in the right spot
            ClearAndResizeBoard (self)
            del self.board
            self.board = copy.deepcopy(cfg.new_board)
            del self.widget_pile_list_counts
            self.widget_pile_list_counts = copy.deepcopy(cfg.new_widget_counts)
            DrawBordersAndBackgrounds (self)
#            print ("DrawL self.wpl ", self.widget_pile_list)
#            print ("DrawL self.wplc ", self.widget_pile_list_counts)

        AddLabels (self)

        # draw game grid
#        print ("draw show board 2", cfg.show_board)
#        print ("board ", self.board)
        y_pos = cfg.img_pix
        x_pos = cfg.img_pix
        self.game_board_x_lower_limit = x_pos
        self.game_board_y_lower_limit = y_pos
#       win_pos = ((self.window_rows - 2) * self.window_cols) + 1
        win_pos = self.window_cols + 1

        for x in range(cfg.game_rows):
            x_pos = self.game_board_x_lower_limit
            for y in range(cfg.game_cols):
                board_position = (cfg.game_cols * x) + y
#                print ("BP WP x y: ", board_position, win_pos, x, y)
                self.fixed_board_sprites.append( pyglet.sprite.Sprite( self.game_bg_image, batch=self.fixed_board_batch, x = x_pos, y = y_pos))
                image = self.spr_mv_list[self.board[board_position][0]][1]
                self.board_sprites.append( pyglet.sprite.Sprite( image, batch=self.variable_board_batch, x = x_pos, y = y_pos))
                image = self.spr_mv_list[self.board[board_position][1]][1]
                self.guess_sprites.append( pyglet.sprite.Sprite( image, batch=self.variable_guess_batch, x = x_pos, y = y_pos))
                self.guess_active_squares.append(win_pos)
                self.guess_active_squares_position.append([x_pos,y_pos])
                self.board_to_window_index.append(win_pos)
                for z in range(len(self.marker_images)):
                    self.marker_sprite = pyglet.sprite.Sprite( self.marker_images[z], batch=self.marker_batch, x = x_pos, y = y_pos)
                    self.marker_sprite.visible = False
                    self.marker_sprites.append(self.marker_sprite)
                x_pos += cfg.img_pix
                win_pos += 1
            y_pos += cfg.img_pix
            win_pos += (cfg.control_cols + 3)
        self.game_board_x_upper_limit = x_pos
        self.game_board_y_upper_limit = y_pos
        cfg.show_board = 1

#        print ("Guess active squares", self.guess_active_squares)
#        print ("Guess active square positions", self.guess_active_squares_position)
#        print ("board_to_window_index", self.board_to_window_index)
#        print ("game board limits ", self.game_board_x_lower_limit, self.game_board_x_upper_limit,
#            self.game_board_y_lower_limit, self.game_board_y_upper_limit)
#        print ("board ", self.board)
#        print ("widget_pile_list_counts ", self.widget_pile_list_counts)

    elif (cfg.show_board == 0):
        for j in range(len(self.board_sprites)):
            self.board_sprites[j].visible = True
        for j in range(len(self.guess_sprites)):
            self.guess_sprites[j].visible = False
#        print ("draw show board 0", len(self.board_sprites), len(self.guess_sprites), cfg.show_board)
    else:
        for j in range(len(self.board_sprites)):
            self.board_sprites[j].visible = False
        for j in range(len(self.guess_sprites)):
            self.guess_sprites[j].visible = True
#        print ("draw show board 1", len(self.board_sprites), len(self.guess_sprites), cfg.show_board)





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
# Copyright (c) Ant <ant@anthive.com>

import pyglet
from pyglet.window import mouse
import random
from random import randrange, getrandbits
from time import sleep
import copy

import ngfp.config as cfg


def InitRandomBoardItems (self):

    # these will come from configuration file eventually...
    gridx = cfg.game_cols
    gridy = cfg.game_rows
    density = cfg.density
    densityfuzz = cfg.density_fuzz
    class_sum = sum(cfg.class_weights)

#    print ("cfg.class_weights, class_sum", cfg.class_weights, class_sum)

    self.class_list = [
                  ["ClSimple", []],
                  ["ClFlipper", []],
                  ["ClBoxSink", []],
                  ["ClAxis", []],
                  ["ClRotator", []],
                  ["ClOneWay", []],
                  ["ClEvil", []]
                 ]

    randpop = ((density * self.board_squares / 100) + 1) // 1
    randpop += ((random.getrandbits(32) % (randpop*densityfuzz*2/100 + 1)) - (randpop*densityfuzz/100)) // 1

    if (randpop > self.board_squares):
        randpop = self.board_squares

    if randpop < 0:
        randpop = 0

    while (randpop > 0):
        position = random.getrandbits(32) % self.board_squares
#            print ("randpop, position", randpop, position)

        if (self.board[position][0] == 0):
            randchance = random.getrandbits(32) % class_sum

#                print ("  randchance", randchance)
            upperbound = 0
            tempobj = 0

            for counter in range(len(cfg.class_weights)):
                upperbound += cfg.class_weights[counter]
#                    print ("    upperbound", upperbound)
                if (upperbound == 0):
                    continue                                           #  Base case

                if (randchance < upperbound):
                    if (self.class_list[counter][0] == "ClSimple"):
                        tempobj = (random.getrandbits(32) % 2) + 1     # 1-2
                        if (self.board[position][0] == 0):
                            self.widget_pile_list_counts[tempobj-1] += 1
                    elif (self.class_list[counter][0] == "ClFlipper"):
                        tempobj = (random.getrandbits(32) % 6) + 3     # 3-8
                        if (self.board[position][0] == 0):
                            if ((tempobj == 3) or (tempobj == 4)):
                                self.widget_pile_list_counts[2] += 1
                            else:
                                self.widget_pile_list_counts[3] += 1
                    elif (self.class_list[counter][0] == "ClBoxSink"):
                        tempobj = (random.getrandbits(32) % 2) + 9     # 9-10
                        if (self.board[position][0] == 0):
                            self.widget_pile_list_counts[tempobj-5] += 1
                    elif (self.class_list[counter][0] == "ClAxis"):
                        tempobj = (random.getrandbits(32) % 4) + 11    # 11-14
                        if (self.board[position][0] == 0):
                            if (tempobj == 11):
                                self.widget_pile_list_counts[6] += 1
                            elif (tempobj == 12):
                                self.widget_pile_list_counts[7] += 1
                            else:
                                self.widget_pile_list_counts[8] += 1
                    elif (self.class_list[counter][0] == "ClRotator"):
                        tempobj = (random.getrandbits(32) % 4) + 15    # 15-18
                        if (self.board[position][0] == 0):
                            if (tempobj == 15):
                                self.widget_pile_list_counts[9] += 1
                            elif (tempobj == 16):
                                self.widget_pile_list_counts[10] += 1
                            else:
                                self.widget_pile_list_counts[11] += 1
                    elif (self.class_list[counter][0] == "ClOneWay"):
                        tempobj = (random.getrandbits(32) % 4) + 19    # 19-22
                        if (self.board[position][0] == 0):
                            self.widget_pile_list_counts[tempobj-7] += 1
                    elif (self.class_list[counter][0] == "ClEvil"):
                        tempobj = (random.getrandbits(32) % 10) + 23   # 23-32
                        if (self.board[position][0] == 0):
                            if ((tempobj >= 23) and (tempobj <= 26)):
                                self.widget_pile_list_counts[16] += 1
                            elif ((tempobj >= 27) and (tempobj <= 30)):
                                self.widget_pile_list_counts[17] += 1
                            elif (tempobj == 31):
                                self.widget_pile_list_counts[18] += 1
                            else:
                                self.widget_pile_list_counts[19] += 1
                    else:
                        pass

                    if (self.board[position][0] == 0):
                        self.board[position][0] = tempobj
#                            print ("    position, tempobj", position, tempobj)
                        self.class_list[counter][1].append([position, tempobj])
                    else:
                        continue

        randpop -= 1

#    print (self.class_list)
#    print (self.widget_pile_list_counts)
#    print (self.board)





#!/usr/bin/env python3 
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
# Copyright (c) Ant <ant@anthive.com>

import pyglet
import sys

import ngfp.config as cfg

from ngfp.dialog import DoDialogControlAction
from ngfp.marbles import DoLeftClickWhiteAction


def WidgetPutClear (self):

    self.picked_up = False
    self.picked_up_from = 0
    self.picked_up_window_square = -1
    self.picked_up_widget = 0
    self.picked_up_widget_index = 0
    self.picked_up_sprite.image = self.white_bg_image
    self.picked_up_sprite.x = 0
    self.picked_up_sprite.y = 0
    self.picked_up_sprite.visible = False

    # put the cube home
    self.cube.x = 0
    self.cube.y = 0
    self.cube.visible = True


def WidgetPick (self, x, x_rec, y, y_rec, win_pos, pick_from, widget_index, widget_spr_index):

    self.picked_up = True
    self.picked_up_from = pick_from
    self.picked_up_window_square = win_pos
    self.picked_up_widget = widget_spr_index
    self.picked_up_widget_index = widget_index
    self.picked_up_sprite.image = self.spr_mv_list[widget_spr_index][1]
    self.picked_up_sprite.x = x - cfg.half_img_pix - 4
    self.picked_up_sprite.y = y - cfg.half_img_pix - 4
    self.picked_up_sprite.visible = True

    # something in progress hide cube
    self.cube.visible = False


def DoLeftClickWidgetAction (self, x, x_rec, y, y_rec, win_pos):

    widget_index = self.widget_active_squares.index(win_pos)
    widget_spr_index = self.widget_pile_list[widget_index]
#    print ("widget_index ", widget_index, " what is count?", self.widget_pile_list_counts[widget_index])
#    print ("widget_spr_index ", widget_spr_index)
    if ((self.picked_up_from == 0) or (self.picked_up_from == 1)):
        if (self.widget_pile_list_counts[widget_index] == 0):
#            print ("Widget count is zero, nothing to do...")
            pass
        elif (self.picked_up == True):
            if (win_pos == self.picked_up_window_square):

                WidgetPutClear (self)
#                print ("Dropped ", widget_index)
            else:

                WidgetPick (self, x, x_rec, y, y_rec, win_pos, 1, 
                    widget_index, widget_spr_index)
#                print ("Picked up other widget ", widget_index)
        else:
            WidgetPick (self, x, x_rec, y, y_rec, win_pos, 1, 
                widget_index, widget_spr_index)
#            print ("Picked up ", widget_index)
    elif (self.picked_up_from == 2):
#        print ("widget to put back", self.picked_up_widget)
        WidgetPutClear (self)


def GuessPut (self, x, x_rec, y, y_rec, win_pos, board_index):

    self.guess_sprites[board_index].image = self.spr_mv_list[self.picked_up_widget][1]
    self.board[board_index][1] = self.picked_up_widget
    if (self.picked_up_from == 1):
        self.widget_pile_list_counts[self.picked_up_widget_index] -= 1

    WidgetPutClear (self)


def GuessPick (self, x, x_rec, y, y_rec, win_pos, board_index):

    self.picked_up = True
    self.picked_up_from = 2
    self.picked_up_window_square = win_pos
    self.picked_up_widget = self.board[board_index][1]
    self.picked_up_sprite.image = self.guess_sprites[board_index].image
    self.guess_sprites[board_index].image = self.game_bg_image
    self.board[board_index][1] = 0
    self.picked_up_widget_index = self.widget_lookup_table[self.picked_up_widget] - 1
    self.widget_pile_list_counts[self.picked_up_widget_index] += 1
    self.picked_up_sprite.x = x - cfg.half_img_pix - 4
    self.picked_up_sprite.y = y - cfg.half_img_pix - 4
    self.picked_up_sprite.visible = True


def DoLeftClickGuessAction (self, x, x_rec, y, y_rec, win_pos):

    guess_board_index = self.board_to_window_index.index(win_pos)
#    print ("Clicked on square ", guess_board_index)

    if (self.picked_up_from == 1):
        if (self.board[guess_board_index][1] != 0):
#            print ("Guess board square not empty...")
            pass
        else:
            GuessPut (self, x, x_rec, y, y_rec, win_pos, guess_board_index)
#            print ("Put it on the board")
    else:
        if (self.board[guess_board_index][1] == 0):
#            print ("Guess board square is empty.  Nothing to move or remove...")
            pass
        else:
            GuessPick (self, x, x_rec, y, y_rec, win_pos, guess_board_index)
            self.picked_up_from = 1
#            print ("Moving or Removing...")


def DoLeftClickControlAction (self, x, x_rec, y, y_rec, win_pos):

    # lets do all of this in the dialog module
    DoDialogControlAction (self, x, x_rec, y, y_rec, win_pos)


def ActiveAreaLeftMouseClickAction (self, x, x_rec, y, y_rec, win_pos):

    cfg.square = win_pos
    if (cfg.show_board == 1):
        if (win_pos in self.white_active_squares):
#            print ("selected ", win_pos, " which is an active White square.")
            DoLeftClickWhiteAction (self, x, x_rec, y, y_rec, win_pos)
        elif (win_pos in self.widget_active_squares):
#            print ("selected ", win_pos, " which is an active Widget square.")
            DoLeftClickWidgetAction (self, x, x_rec, y, y_rec, win_pos)
        elif (win_pos in self.guess_active_squares):
#            print ("selected ", win_pos, " which is an active Guess square.")
            DoLeftClickGuessAction (self, x, x_rec, y, y_rec, win_pos)
        elif (win_pos in self.control_active_squares):
#            print ("selected ", win_pos, " which is an active Control square.")
            DoLeftClickControlAction (self, x, x_rec, y, y_rec, win_pos)
        else:
            pass


def DoGuessRotateWidgetAction (self, x, x_rec, y, y_rec, win_pos):

    board_index = self.board_to_window_index.index(win_pos)
    picked_widget = self.board[board_index][1]
    if (picked_widget == 0):
#        print ("Nothing there to rotate...")
        pass
    else:
        widget_lut = self.widget_lookup_table[picked_widget]
        widget_base = self.widget_pile_list[widget_lut-1]
        ft_wrm = self.widget_rotate_modulus[widget_lut]

#        print ("Before ==>\nboard_index ", board_index,
#               "widget_base", widget_base,
#               "picked_widget ", picked_widget,
#               "widget_lut ", widget_lut,
#               "ft_wrm ", ft_wrm
#              )
        if (ft_wrm == 0):
#            print ("Can't rotate that one...")
            return
        elif ((picked_widget + 1) >= (widget_base + ft_wrm)):
#            print ("roll around ")
            picked_widget = widget_base
        else:
#            print ("show next in sequence ")
            picked_widget += 1

#        print ("After ==>\nboard_index ", board_index,
#               "widget_base", widget_base,
#               "picked_widget ", picked_widget,
#               "widget_lut ", widget_lut,
#               "ft_wrm ", ft_wrm
#              )

        self.board[board_index][1] = picked_widget
        self.guess_sprites[board_index].image = self.spr_mv_list[picked_widget][1]
#        print ("lut ", self.widget_lookup_table)
#        print ("wpl ", self.widget_pile_list)
#        print ("wrm ", self.widget_rotate_modulus)


def ActiveAreaRightMouseClickAction (self, x, x_rec, y, y_rec, win_pos):

    cfg.square = win_pos
    if (cfg.show_board == 1):
        if (win_pos in self.white_active_squares):
#            print ("selected ", win_pos, " which is an active White square.")
            pass
        elif (win_pos in self.widget_active_squares):
#            print ("selected ", win_pos, " which is an active Widget square.")
            pass
        elif (win_pos in self.guess_active_squares):
#            print ("selected ", win_pos, " which is an active Guess square.")
            DoGuessRotateWidgetAction (self, x, x_rec, y, y_rec, win_pos)
        else:
            pass


def ActiveAreaMouseMoveAction (self, x, x_rec, y, y_rec, win_pos):

    if (cfg.show_board == 1):
        if (win_pos in self.widget_active_squares):
            if (cfg.square != win_pos):
#                print ("moved over ", win_pos, " which is an active Widget square.")
                cfg.square = win_pos
            self.picked_up_sprite.x = x - cfg.half_img_pix - 4
            self.picked_up_sprite.y = y - cfg.half_img_pix - 4
            self.picked_up_sprite.visible = True
        elif (win_pos in self.guess_active_squares):
            if (cfg.square != win_pos):
#                print ("moved over ", win_pos, " which is an active Guess square.")
                cfg.square = win_pos
            self.picked_up_sprite.x = x - cfg.half_img_pix - 4
            self.picked_up_sprite.y = y - cfg.half_img_pix - 4
            self.picked_up_sprite.visible = True
        else:
            self.picked_up_sprite.x = x_rec
            self.picked_up_sprite.y = y_rec
            self.picked_up_sprite.visible = False
    else:
        pass





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
# Copyright (c) Ant <ant@anthive.com>

import pyglet
import copy

import ngfp.config as cfg


def AddLabels (self):

    w_limit = len(self.widget_pile_list_counts)-1
    w = 0

#    print ("WL len ", len(self.widget_labels))
#    print ("WPLC len ", len(self.widget_pile_list_counts))

    if (len(self.widget_labels) != 0):
#        print ("Removing Existing Widget Labels")
        for j in range(len(self.widget_labels)):
            self.widget_labels[j].visible = False
            self.widget_labels[j].delete()
        del self.widget_labels
        self.widget_labels = []

    y_pos = cfg.img_pix * (cfg.game_rows)
    for i in range(cfg.game_rows+1):
        x_pos = cfg.img_pix * (cfg.game_cols+3)
        for j in range(cfg.control_cols):
            if ((i % 2) == 0):
                if (w < w_limit):
                    count_str = str(self.widget_pile_list_counts[w])
                    self.widget_labels.append(pyglet.text.Label(
                        count_str,
                        font_name="Sans Regular",
                        font_size=28,
                        bold=True,
                        color=[245, 0, 0, 255],
                        width=cfg.img_pix, height=cfg.img_pix,
                        x=x_pos+34, y=y_pos+24,
                        anchor_x="center", anchor_y="center",
                        batch=self.text_batch))
                    w += 1
                x_pos += cfg.img_pix
        y_pos -= cfg.img_pix


def UpdateLabels (self):

#    print ("UPDTL WL len ", len(self.widget_labels))
#    print ("UPDTL WPLC len ", len(self.widget_pile_list_counts))

    for w in range(len(self.widget_labels)):
        self.widget_labels[w].text = str(self.widget_pile_list_counts[w])





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
# Copyright (c) Ant <ant@anthive.com>

import copy
import os
import random

import pyglet

import ngfp.config as cfg

from ngfp.dialog import LoadConfigOrUseCurrent


def MyInitStuff (self):

    random.seed()

    self.set_visible(False)

    self.png_path = cfg.png_path

    # screens, sizes and locations
    #   some of these change as the board changes size
    self.top_display = pyglet.canvas.get_display()
    self.top_screen = self.top_display.get_default_screen()
    self.full_screen_width = self.top_screen.width
    self.full_screen_height = self.top_screen.height
#    print ("Initial Window Size : ", self.full_screen_width, self.full_screen_height)
    self.windows_lst = self.top_display.get_windows()
    self.screen_width = self.windows_lst[0].width
    self.screen_height = self.windows_lst[0].height
    self.x, self.y = self.windows_lst[0].get_location()
#    print ("Smaller Window Location and Size : ", self.x, self.y, self.screen_width, self.screen_height)

    # initial window is blank and flickers i don't want 
    # to see it until it is resized later
    self.windows_lst[0].set_visible(False)

    # if there's a config file use it
    #  if there isn't set defaults specified in config.py
    LoadConfigOrUseCurrent ()

    # other useful constants
    self.board_squares = cfg.game_rows*cfg.game_cols
    self.window_cols = (cfg.game_cols+cfg.control_cols+3)
    self.window_rows = (cfg.game_rows+2)
    self.window_squares = self.window_rows*self.window_cols

    self.game_board_x_limit = 0
    self.game_board_y_limit = 0

    self.keys_held = []
    self.key = pyglet.window.key

    self.mouse_win_pos = 0

    self.fps = pyglet.window.FPSDisplay(self)

    # batches for rendering
    self.fixed_batch = pyglet.graphics.Batch()
    self.green_batch = pyglet.graphics.Batch()
    self.control_batch = pyglet.graphics.Batch()
    self.widget_batch = pyglet.graphics.Batch()
    self.fixed_board_batch = pyglet.graphics.Batch()
    self.variable_board_batch = pyglet.graphics.Batch()
    self.variable_guess_batch = pyglet.graphics.Batch()
    self.pointer_bottom_batch = pyglet.graphics.Batch()
    self.pointer_top_batch = pyglet.graphics.Batch()
    self.text_batch = pyglet.graphics.Batch()
    self.arrow_batch = pyglet.graphics.Batch()
    self.marble_batch = pyglet.graphics.Batch()
    self.marker_batch = pyglet.graphics.Batch()

    # colors need precedence order for arrows
    #   we only use as many colors to mark arrows we keep in history
    self.history_limit = 6
    self.color_batch_list = []
    for i in range(self.history_limit):
        self.color_batch_list.append(pyglet.graphics.Batch())

    # lists of sprites
    self.fixed_sprites = []
    self.fixed_board_sprites = []
    self.green_sprites = []
    self.board_sprites = []
    self.guess_sprites = []
    self.control_sprites = []
    self.widget_sprites = []
    self.top_sprites = []
    self.arrow_history_sprites = []
    self.history_color_sprites = []
    self.marble_sprites = []
    self.marker_sprites = []

    self.white_active_squares = []
    self.white_active_squares_position = []
    self.guess_active_squares = []
    self.guess_active_squares_position = []
    self.control_active_squares = []
    self.control_active_squares_position = []
    self.widget_active_squares = []
    self.widget_active_squares_position = []
    self.board_to_window_index = []

    self.game_bg_image  = pyglet.image.load(self.png_path + "mirrors/00_bg.png")
    self.white_bg_image = pyglet.image.load(self.png_path + "backgrounds/wbg.png")
    self.blue_bg_image  = pyglet.image.load(self.png_path + "backgrounds/bbg.png")
    self.green_bg_image = pyglet.image.load(self.png_path + "backgrounds/lgbg.png")
    self.gray_bg_image  = pyglet.image.load(self.png_path + "backgrounds/gbg.png")

    self.gcube_image = pyglet.image.load(self.png_path + "misc/gcube.png")
    self.cube_image  = pyglet.image.load(self.png_path + "misc/cube.png")

    self.pic_marker_list = [
        self.png_path + "markers/picMkCircle.png",
        self.png_path + "markers/picMkTriangle.png",
        self.png_path + "markers/picMkSquare.png",
        self.png_path + "markers/picMkVee.png"
        ]

    self.marker_images = []
    for i in range(len(self.pic_marker_list)):
        self.marker_images.append(pyglet.image.load(self.pic_marker_list[i]))

    self.pic_control_list = [
        self.png_path + "controls/picINew.png",
        self.png_path + "controls/picICheck.png",
        self.png_path + "controls/picIFlipBoards.png",
        self.png_path + "controls/picIOpen.png",
        self.png_path + "controls/picISave.png",
        self.png_path + "controls/picIAbout.png"
        ]

    self.control_images = []
    for i in range(len(self.pic_control_list)):
        self.control_images.append(pyglet.image.load(self.pic_control_list[i]))

    self.pic_marbles_list = [
        self.png_path + "marbles/red_marbles.png",
        self.png_path + "marbles/green_marbles.png",
        self.png_path + "marbles/blue_marbles.png",
        self.png_path + "marbles/yellow_marbles.png",
        self.png_path + "marbles/purple_marbles.png",
        self.png_path + "marbles/black_marbles.png"
        ]

    self.marble_images = []
    for i in range(len(self.pic_marbles_list)):
        self.marble_images.append(pyglet.image.load(self.pic_marbles_list[i]))

    self.pic_color_list = [
        self.png_path + "colors/red_half.png",
        self.png_path + "colors/green_half.png",
        self.png_path + "colors/blue_half.png",
        self.png_path + "colors/yellow_half.png",
        self.png_path + "colors/purple_half.png",
        self.png_path + "colors/black_half.png"
        ]

    self.color_images = []
    for i in range(len(self.pic_color_list)):
        self.color_images.append(pyglet.image.load(self.pic_color_list[i]))

    self.pic_arrow_list = [
        self.png_path + "arrows/picDLeftW.png",
        self.png_path + "arrows/picDRightW.png",
        self.png_path + "arrows/picDUpW.png",
        self.png_path + "arrows/picDDownW.png"
        ]

    self.arrow_images = []
    for i in range(len(self.pic_arrow_list)):
        self.arrow_images.append(pyglet.image.load(self.pic_arrow_list[i]))

    # arrow history list and current index, set up colors and marbles too...
    self.arrow_index = 0
    self.anim = []
    self.marble_seq = []
    for i in range(self.history_limit):

        spr_a = pyglet.sprite.Sprite(self.arrow_images[0], batch=self.arrow_batch)
        spr_a.visible = False
        spr_b = pyglet.sprite.Sprite(self.arrow_images[0], batch=self.arrow_batch)
        spr_b.visible = False
        self.arrow_history_sprites.append([spr_a, spr_b])

        spr_c = pyglet.sprite.Sprite(self.color_images[i], batch=self.color_batch_list[i])
        spr_c.visible = False
        spr_d = pyglet.sprite.Sprite(self.color_images[i], batch=self.color_batch_list[i])
        spr_d.visible = False
        self.history_color_sprites.append([spr_c, spr_d])

        self.marble_seq.append(pyglet.image.ImageGrid(self.marble_images[i], 1, 16))
        self.anim.append(pyglet.image.Animation.from_image_sequence(self.marble_seq[i], 0.02, True))
        spr_e = pyglet.sprite.Sprite(self.anim[i], batch=self.marble_batch)
        spr_e.visible = True
        spr_e.x = 0
        spr_e.y = 0
        spr_e.dx = 0
        spr_e.dy = 0
        self.marble_sprites.append(spr_e)

    # cfg.pic_list is a copy of this.  i need to change all the rest of
    # the references to this list to cfg.pic_list instead and remove this
    # so it is no longer duplicated...  someday...
    self.pic_list = [
        self.png_path + "mirrors/00_bg.png",           # background
        self.png_path + "mirrors/01_normal.png",       # simple mirrors: left: \
        self.png_path + "mirrors/02_normal.png",       # simple mirrors: right: /
        self.png_path + "mirrors/03_flip2.png",        # simple flipping mirrors: left: \
        self.png_path + "mirrors/04_flip2.png",        # simple flipping mirrors: right: /
        self.png_path + "mirrors/05_flip4.png",        # quad flipping mirrors: left: \
        self.png_path + "mirrors/06_flip4.png",        # quad flipping mirrors: bounce: o
        self.png_path + "mirrors/07_flip4.png",        # quad flipping mirrors: right: /
        self.png_path + "mirrors/08_flip4.png",        # quad flipping mirrors: bounce: o
        self.png_path + "mirrors/09_block.png",        # box and sink: box: bounce: o  (reflect all)
        self.png_path + "mirrors/10_sink.png",         # box and sink: sink: grab: x  (absorb all)
        self.png_path + "mirrors/11_axial.png",        # axial mirrors: simple vertical: |
        self.png_path + "mirrors/12_axial.png",        # axial mirrors: simple horizontal: -
        self.png_path + "mirrors/13_axial2.png",       # axial mirrors: flipping vertical: ||
        self.png_path + "mirrors/14_axial2.png",       # axial mirrors: flipping horizontal: =
        self.png_path + "mirrors/15_rotator.png",      # rotators simple counterclockwise: left: \\
        self.png_path + "mirrors/16_rotator.png",      # rotators simple clockwise: right: //
        self.png_path + "mirrors/17_rotator2.png",     # rotators flipper clockwise: left: []
        self.png_path + "mirrors/18_rotator2.png",     # rotators flipper counterclockwise: right: ][
        self.png_path + "mirrors/19_half.png",         # 1-way mirrors: left: lower reflects: \<-
        self.png_path + "mirrors/20_half.png",         # 1-way mirrors: left: upper reflects: ->\
        self.png_path + "mirrors/21_half.png",         # 1-way mirrors: right: lower reflects: ->/
        self.png_path + "mirrors/22_half.png",         # 1-way mirrors: right: upper reflects: /<-
        self.png_path + "mirrors/23_half4.png",        # flipping 1-way mirrors: left: lower reflects: rotates clockwise: \\\<-
        self.png_path + "mirrors/24_half4.png",        # flipping 1-way mirrors: right: upper reflects: rotates clockwise: ///<-
        self.png_path + "mirrors/25_half4.png",        # flipping 1-way mirrors: left: upper reflects: rotates clockwise: ->\\\
        self.png_path + "mirrors/26_half4.png",        # flipping 1-way mirrors: right: lower reflects: rotates clockwise: ->///
        self.png_path + "mirrors/27_half4.png",        # flipping 1-way mirrors: left: upper reflects: rotates counterclockwise: ->\\\
        self.png_path + "mirrors/28_half4.png",        # flipping 1-way mirrors: right: upper reflects: rotates counterclockwise: ///<-
        self.png_path + "mirrors/29_half4.png",        # flipping 1-way mirrors: left: lower reflects: rotates counterclockwise: \\\<-
        self.png_path + "mirrors/30_half4.png",        # flipping 1-way mirrors: right: lower reflects: rotates counterclockwise: ->///
        self.png_path + "mirrors/31_move.png",         # moving mirror: left: -->\X\--> ---->\X\
        self.png_path + "mirrors/32_move.png",         # moving mirror: right: <--/X/<-- /X/<----
        self.png_path + "mirrors/33_bg.png"            # background
        ]

    self.spr_mv_list = []

    for i in range(len(self.pic_list)):
        image = pyglet.image.load(self.pic_list[i])
        sprite = pyglet.sprite.Sprite(image)
        self.spr_mv_list.append([0, image, sprite, 0, 0])

    # these flags are for the widget pile list
    #     position is taken from widget list location index
    #     when self.widget_pile_list_counts[position] != 0 then the index
    #        can be used to get the number from this list to index into
    #         self.spr_mv_list for the image or sprite or ...
    self.widget_pile_list = [1,2,3,5,9,10,11,12,13,15,16,17,19,20,21,22,23,27,31,32]
    for i in self.widget_pile_list:
        self.spr_mv_list[i][0] = 1
    self.widget_pile_list_counts = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
    self.widget_labels = []

    # eventually we're going to have to index any widget no matter how it
    # is rotated...
    self.widget_lookup_table = [0, 1, 2, 
                                3, 3,
                                4, 4, 4, 4,
                                5, 6, 7, 8,
                                9, 9,
                                10, 11,
                                12, 12,
                                13, 14, 15, 16,
                                17, 17, 17, 17,
                                18, 18, 18, 18,
                                19, 20
                               ]

    # speaking of rotating, we need to know how many places it 
    #   rotates through - of course 0 means it doesn't rotate at all
    # note that this aligns with self.widget_lookup_table
    self.widget_rotate_modulus = [0,0,0,2,4,0,0,0,0,2,0,0,2,0,0,0,0,4,4,0,0]

    # this says what the next widget will be in the sequence if it isn't 0
    self.widget_next_widget = [
    #   0 1 2 3 4 5 6 7 8 9 0 1 2
        0,0,0,4,3,6,7,8,5,0,0,0,0,
        14,13,0,0,18,17,
        0,0,0,0,
    #   23-26 
        24, 25, 26, 23,
    #   27-30
        28, 29, 30, 27,
    #   31, 32
        0, 0]

    # these flags are for the configuration of each group percentages images list
    # even numbers are left image indexes, odd are right, seven groups of two
    self.config_percent_list = [1,2,3,5,9,10,11,14,15,18,19,22,23,31]
    for i in self.config_percent_list:
        self.spr_mv_list[i][3] = 1

    self.property_labels = [
                            "Width",
                            "Height",
                            "Density",
                            "Fuzz"
                           ]

    self.widget_cl_labels = [
                          "Simple mirrors",
                          "Flipping mirrors",
                          "Box and sink",
                          "Axial mirrors",
                          "Rotators",
                          "One-way mirrors",
                          "PURE EVIL"
                         ]

    # 0 = nowhere, 1 = widget, 2 = board
    self.picked_up_from = 0
    self.picked_up = False

    # direction sets will make things easier later
    self.dir_left = []
    self.dir_right = []
    self.dir_up = []
    self.dir_down = []

    # some kind of limit to break out of loops
    self.tick_limit = 100 + (self.board_squares * 2)





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
# Copyright (c) Ant <ant@anthive.com>

from os import path
from setuptools import setup, find_packages


# read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()


setup(
    name="ngfp",
    version="0.1.6",
    author="Ant",
    author_email="ant@anthive.com",
    description="A puzzle game based upon gfpoken.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://salsa.debian.org/ant-guest/gfpoken-in-python",
    license="Apache-2.0",
    setup_requires=["setuptools >= 40.6.3",
                    "twine >= 1.12.1",
                    "wheel >= 0.32.3"],
    packages=find_packages(),
    install_requires=["pyglet >= 1.3.0",
                      "pygobject"],
    provides=["ngfp"],
    include_package_data=True,
    entry_points={
        "console_scripts": ["runngfp = ngfp.ngfp:main"]
                 },
    python_requires=">=3",
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python :: 3",
        "Operating System :: POSIX :: Linux",
        "Intended Audience :: End Users/Desktop",
        "Topic :: Games/Entertainment :: Puzzle Games"
    ],
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
# Copyright (c) Ant <ant@anthive.com>

import pyglet
import copy

import ngfp.config as cfg


def DrawBordersAndBackgrounds (self):

    # draw four blue corner squares
    y_pos = 0
    x_pos = 0
    x_pos_right = cfg.img_pix * (cfg.game_cols+1)
    self.fixed_sprites.append( pyglet.sprite.Sprite( self.blue_bg_image, batch=self.fixed_batch, x = x_pos, y = y_pos))
    self.fixed_sprites.append( pyglet.sprite.Sprite( self.blue_bg_image, batch=self.fixed_batch, x = x_pos_right, y = y_pos))
    y_pos = cfg.img_pix * (cfg.game_rows+1)
    self.fixed_sprites.append( pyglet.sprite.Sprite( self.blue_bg_image, batch=self.fixed_batch, x = x_pos, y = y_pos))
    self.fixed_sprites.append( pyglet.sprite.Sprite( self.blue_bg_image, batch=self.fixed_batch, x = x_pos_right, y = y_pos))

    # draw white game border
    self.white_active_squares = []
    self.white_active_squares_position = []
    y_pos = cfg.img_pix
    x_pos = 0
    x_pos_right = cfg.img_pix * (cfg.game_cols+1)
    for y in range(cfg.game_rows):
        self.fixed_sprites.append( pyglet.sprite.Sprite( self.white_bg_image, batch=self.fixed_batch, x = x_pos, y = y_pos))
        self.white_active_squares.append((self.window_cols * (y_pos // cfg.img_pix)))
        self.white_active_squares_position.append([x_pos,y_pos])
        self.dir_left.append((self.window_cols * (y_pos // cfg.img_pix)))
        self.fixed_sprites.append( pyglet.sprite.Sprite( self.white_bg_image, batch=self.fixed_batch, x = x_pos_right, y = y_pos))
        self.white_active_squares.append((self.window_cols * (y_pos // cfg.img_pix))+cfg.game_cols+1)
        self.white_active_squares_position.append([x_pos_right,y_pos])
        self.dir_right.append((self.window_cols * (y_pos // cfg.img_pix))+cfg.game_cols+1)
        y_pos += cfg.img_pix
    y_pos = 0
    x_pos = cfg.img_pix
    y_pos_up = cfg.img_pix * (cfg.game_rows+1)
    for x in range(cfg.game_cols):
        self.fixed_sprites.append( pyglet.sprite.Sprite( self.white_bg_image, batch=self.fixed_batch, x = x_pos, y = y_pos))
        self.white_active_squares.append(x+1)
        self.white_active_squares_position.append([x_pos,y_pos])
        self.dir_down.append(x+1)
        self.fixed_sprites.append( pyglet.sprite.Sprite( self.white_bg_image, batch=self.fixed_batch, x = x_pos, y = y_pos_up))
        self.white_active_squares.append(((self.window_rows-1) * self.window_cols)+x+1)
        self.white_active_squares_position.append([x_pos,y_pos_up])
        self.dir_up.append(((self.window_rows-1) * self.window_cols)+x+1)
        x_pos += cfg.img_pix
#    print ("White active squares", self.white_active_squares)
#    print ("White active square positions", self.white_active_squares_position)
#    print ("Left white active squares", self.dir_left)
#    print ("Right white active squares", self.dir_right)
#    print ("Up white active squares", self.dir_up)
#    print ("Down white active squares", self.dir_down)

    # draw vertical green controls
    self.control_active_squares = []
    self.control_active_squares_position = []
    x_pos = cfg.img_pix * (cfg.game_cols+2)
    y_pos = cfg.img_pix * (cfg.game_rows+1)
    win_pos = self.window_squares - (cfg.control_cols + 1)
    control_counter = 0
    for y in range(cfg.game_rows+2):
        self.green_sprites.append( pyglet.sprite.Sprite( self.green_bg_image, batch=self.green_batch, x = x_pos, y = y_pos))
        if (control_counter in range(len (self.control_images))):
            self.control_sprites.append( pyglet.sprite.Sprite( self.control_images[control_counter], batch=self.control_batch, x = x_pos, y = y_pos))
            self.control_active_squares.append(win_pos)
            self.control_active_squares_position.append([x_pos,y_pos])
            win_pos -= self.window_cols
            control_counter += 1
        y_pos -= cfg.img_pix
#    print ("Control active squares", self.control_active_squares)
#    print ("Control active square positions", self.control_active_squares_position)

    # put down the widget piles.  for any empty spots use 
    #    the light gray background.  alternate rows of
    #    widgets and gray background.
    # 
    # first get the list of which ones and put them on 
    #   their own list.
    widget_list = copy.deepcopy(self.widget_pile_list)

#    print ("Background WL ", widget_list)

    # ignore the extra one on the end
    spr_length = (len(self.spr_mv_list) - 1)

    self.widget_active_squares = []
    self.widget_active_squares_position = []
    y_pos = cfg.img_pix * (cfg.game_rows+1)
    win_pos = self.window_squares - cfg.control_cols
    for x in range(cfg.game_rows+2):
        x_pos = cfg.img_pix * (cfg.game_cols+3)
        for y in range(cfg.control_cols):
            if ((x % 2) == 0):
                try:
                    spr_index = widget_list.pop(0)
                    self.spr_mv_list[spr_index][2].batch = self.widget_batch
                    self.spr_mv_list[spr_index][2].x = x_pos
                    self.spr_mv_list[spr_index][2].y = y_pos
                    self.spr_mv_list[spr_index][2].visible = True
                    self.widget_sprites.append(self.spr_mv_list[spr_index][2])
                    self.widget_active_squares.append(win_pos+y)
                    self.widget_active_squares_position.append([x_pos,y_pos])
#                    print ("Len w_a_s, w_a_s_p ", len(self.widget_active_squares), len(self.widget_active_squares_position))
                except:
                    self.fixed_sprites.append( pyglet.sprite.Sprite( self.gray_bg_image, batch=self.fixed_batch, x = x_pos, y = y_pos))
#                    print ("Background Widget Exception ", x_pos, y_pos)
            else:
                self.fixed_sprites.append( pyglet.sprite.Sprite( self.gray_bg_image, batch=self.fixed_batch, x = x_pos, y = y_pos))

            x_pos += cfg.img_pix
            if spr_index < spr_length:
                spr_index += 1
        y_pos -= cfg.img_pix
        win_pos -= self.window_cols
#    print ("Widget active squares", self.widget_active_squares)
#    print ("Widget active square positions", self.widget_active_squares_position)

    # then fill in the rest of the rows with the 
    # gray background.
#    y_pos = cfg.img_pix * cfg.game_rows
#    for x in range(cfg.game_rows+1):
#        x_pos = cfg.img_pix * (cfg.game_cols+3)
#        for y in range(cfg.control_cols):
#            self.fixed_sprites.append( pyglet.sprite.Sprite( self.gray_bg_image, batch=self.fixed_batch, x = x_pos, y = y_pos))
#            x_pos += cfg.img_pix
#        y_pos -= cfg.img_pix*2


